package com.zanxiang.game.data.serve.service.impl;

import com.google.common.base.CaseFormat;
import com.google.gson.Gson;
import com.zanxiang.game.data.serve.pojo.dto.*;
import com.zanxiang.game.data.serve.pojo.entity.*;
import com.zanxiang.game.data.serve.pojo.enums.OrderByEnum;
import com.zanxiang.game.data.serve.pojo.vo.*;
import com.zanxiang.game.data.serve.service.IGameDataService;
import com.zanxiang.game.data.serve.utils.Page;
import com.zanxiang.module.util.DateUtil;
import com.zanxiang.module.util.exception.BaseException;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.nutz.dao.Cnd;
import org.nutz.dao.Dao;
import org.nutz.dao.Sqls;
import org.nutz.dao.entity.Entity;
import org.nutz.dao.pager.Pager;
import org.nutz.dao.sql.Criteria;
import org.nutz.dao.sql.Sql;
import org.nutz.dao.util.Daos;
import org.nutz.dao.util.cri.SimpleCriteria;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import reactor.util.function.Tuple2;
import reactor.util.function.Tuples;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author tianhua
 * @time 2023/7/12
 * @Description 游戏数据逻辑处理
 **/
@Service
@Slf4j
public class GameDataServiceImpl implements IGameDataService {
    private static final List<Tuple2<Field, Field>> dayNFieldMapList;
    private static final List<Tuple2<Field, Field>> dayNTotalFieldMapList;

    static {
        Map<String, Field> fieldMap = new HashMap<>();
        Map<String, Field> fieldTotalMap = new HashMap<>();
        List<Field> dayNFieldList = new ArrayList<>();
        List<Field> dayNTotalFieldList = new ArrayList<>();
        Class<?> tempClazz = GameDataDayVO.class;
        Class<?> tempTotalClazz = GameDataDayTotalVO.class;

        while (tempClazz != null) {
            Field[] fields = tempClazz.getDeclaredFields();
            for (Field field : fields) {
                if (Modifier.isFinal(field.getModifiers()) || Modifier.isStatic(field.getModifiers())) {
                    continue;
                }
                fieldMap.put(field.getName(), field);
                if (field.getType() == RechargeTrendVO.class) {
                    dayNFieldList.add(field);
                }
            }
            tempClazz = tempClazz.getSuperclass();
        }

        while (tempTotalClazz != null) {
            Field[] fields = tempTotalClazz.getDeclaredFields();
            for (Field field : fields) {
                if (Modifier.isFinal(field.getModifiers()) || Modifier.isStatic(field.getModifiers())) {
                    continue;
                }
                fieldTotalMap.put(field.getName(), field);
                if (field.getType() == RechargeTrendVO.class) {
                    dayNTotalFieldList.add(field);
                }
            }
            tempTotalClazz = tempTotalClazz.getSuperclass();
        }

        if (dayNFieldList.isEmpty()) {
            dayNFieldMapList = Collections.emptyList();
        } else {
            dayNFieldMapList = new ArrayList<>(dayNFieldList.size());
            for (Field dayNField : dayNFieldList) {
                dayNField.setAccessible(true);
                Field sourceField = fieldMap.get(dayNField.getName().replace("Trend", ""));
                sourceField.setAccessible(true);
                if (sourceField != null) {
                    dayNFieldMapList.add(Tuples.of(sourceField, dayNField));
                }
            }
        }

        if (dayNTotalFieldList.isEmpty()) {
            dayNTotalFieldMapList = Collections.emptyList();
        } else {
            dayNTotalFieldMapList = new ArrayList<>(dayNTotalFieldList.size());
            for (Field dayNTotalField : dayNTotalFieldList) {
                dayNTotalField.setAccessible(true);
                Field sourceField = fieldTotalMap.get(dayNTotalField.getName().replace("Trend", ""));
                sourceField.setAccessible(true);
                if (sourceField != null) {
                    dayNTotalFieldMapList.add(Tuples.of(sourceField, dayNTotalField));
                }
            }
        }
    }

    @Autowired
    private Dao dao;

    /**
     * 游戏每日数据
     *
     * @param dto 前端传递查询参数
     * @return 返回给前端的数据
     */
    @Override
    public Page<GameDataDayVO> getGameDataDay(GameDataDayDTO dto) {

        //传入的注册时间为空时，给一个默认值 当天数据
        if (dto.getRegisteredBeginDate() == null || dto.getRegisteredEndDate() == null) {
            dto.setRegisteredBeginDate(LocalDate.now());
            dto.setRegisteredEndDate(LocalDate.now());
        }
        //如果没有排序条件给默认值
        if (StringUtils.isBlank(dto.getSortFiled())) {
            dto.setSortFiled("dt");
        }
        if (StringUtils.isBlank(dto.getSortType())) {
            dto.setSortType(OrderByEnum.DESC.getOrderType());
        }
        //根据dto拼接查询条件
        Criteria cri = Cnd.cri();
        if (StringUtils.isNotBlank(dto.getGameName())) {
            //拼接游戏名称查询条件
            cri.where().andEquals("game_name", dto.getGameName());
        }
        if (dto.getGameId() != null) {
            //拼接游戏id查询条件
            cri.where().andEquals("game_id", dto.getGameId());
        }
        if (dto.getGameClassify() != null) {
            //拼接游戏类型查询条件
            cri.where().andEquals("game_classify", dto.getGameClassify());
        }
        if (dto.getRegisteredBeginDate() != null && dto.getRegisteredEndDate() != null) {
            cri.where().andBetween("dt", dto.getRegisteredBeginDate(), dto.getRegisteredEndDate());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            //拼接SDK来源
            cri.where().andEquals("source_system", dto.getSourceSystem());
        }
        //拼接排序条件
        cri.getOrderBy().orderBy(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, dto.getSortFiled()), dto.getSortType());

        //编写sql语句 拼接查询条件
        Sql sql = Sqls.create(gameDataDaySql() + cri);
        //设置自定义回显对象
        sql.setCallback(Sqls.callback.entities());
        sql.setEntity(dao.getEntity(GameDataDayVO.class));
        //设置pager对象
        Pager pager = dao.createPager(dto.getPageNum(), dto.getPageSize());
        sql.setPager(pager);
        //执行sql
        dao.execute(sql);
        //得到结果集list
        List<GameDataDayVO> list = sql.getList(GameDataDayVO.class);
        //设置查询总数
        pager.setRecordCount(dao.count(AdsGameDay.class, cri));

        List<GameDataDayVO> gameDataDayVOList = list.stream().map(vo -> {
            formatDayN(vo);
            return vo;
        }).collect(Collectors.toList());

        //返回list结果 封装到page对象里
        return new Page<>(gameDataDayVOList, pager);
    }

    /**
     * 游戏每日数据总计
     *
     * @param dto 前端传递的查询条件
     * @return 返回给前端的总计数据实体
     */
    public GameDataDayTotalVO getGameDataDayTotal(GameDataDayTotalDTO dto) {

        //如果没有传入查询时间默认查询当天
        if (dto.getRegisteredBeginDate() == null || dto.getRegisteredEndDate() == null) {
            dto.setRegisteredBeginDate(LocalDate.now());
            dto.setRegisteredEndDate(LocalDate.now());
        }
        //新增查询条件
        Criteria cri = Cnd.cri();
        if (dto.getGameId() != null) {
            //拼接游戏id
            cri.where().andEquals("game_id", dto.getGameId());
        }
        if (StringUtils.isNotBlank(dto.getGameName())) {
            //拼接游戏名称
            cri.where().andEquals("game_name", dto.getGameName());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            //拼接SDK来源
            cri.where().andEquals("source_system", dto.getSourceSystem());
        }
        if (dto.getGameClassify() != null) {
            cri.where().andEquals("game_classify", dto.getGameClassify());
        }
        if (dto.getRegisteredBeginDate() != null && dto.getRegisteredEndDate() != null) {
            //拼接查询时间
            cri.where().andBetween("dt", dto.getRegisteredBeginDate(), dto.getRegisteredEndDate());
        }

        //创建sql
        Sql gameDataDayTotalsql = Sqls.create(gameDataDayTotalSql() + cri);
        //设置自定义回传对象
        gameDataDayTotalsql.setCallback(Sqls.callback.entity());
        gameDataDayTotalsql.setEntity(dao.getEntity(GameDataDayTotalVO.class));
        //执行sql
        dao.execute(gameDataDayTotalsql);
        //得到每日总计对象
        GameDataDayTotalVO vo = gameDataDayTotalsql.getObject(GameDataDayTotalVO.class);
        if (vo.getAmountD1() != null) {
            formatDayNTotal(vo);
        }
        return vo;
    }

    /**
     * 游戏总数据
     *
     * @param dto 前端传递的查询参数
     * @return 返回给前端的数据
     */
    @Override
    public Page<GameDataTotalVO> getGameDataTotal(GameDataTotalDTO dto) {

        //如果注册时间参数为空，默认设置查询当天数据
        if (dto.getRegisteredBeginDate() == null || dto.getRegisteredEndDate() == null) {
            dto.setRegisteredBeginDate(LocalDate.now());
            dto.setRegisteredEndDate(LocalDate.now());
        }
        //如果充值时间参数为空，默认设置查询当天数据
        if (dto.getRechargeBeginDate() == null || dto.getRechargeEndDate() == null) {
            dto.setRechargeBeginDate(LocalDate.now());
            dto.setRechargeEndDate(LocalDate.now());
        }
        //如果没有排序条件给默认值
        if (StringUtils.isBlank(dto.getSortFiled())) {
            //根据游戏id
            dto.setSortFiled("game_id");
        }
        if (StringUtils.isBlank(dto.getSortType())) {
            dto.setSortType(OrderByEnum.ASC.getOrderType());
        }
        //根据传入的dto拼接查询参数
        Criteria cri = Cnd.cri();
        if (StringUtils.isNotBlank(dto.getGameName())) {
            //拼接游戏名称查询条件
            cri.where().andEquals("a.game_name", dto.getGameName());
        }
        if (dto.getGameId() != null) {
            //拼接游戏id查询条件
            cri.where().andEquals("game_id", dto.getGameId());
        }
        if (dto.getGameClassify() != null) {
            //拼接游戏类型查询条件
            cri.where().andEquals("a.game_classify", dto.getGameClassify());
        }
        if (dto.getRegisteredBeginDate() != null && dto.getRegisteredEndDate() != null) {
            //拼接注册日期查询条件
            cri.where().andBetween("a.dt", dto.getRegisteredBeginDate(), dto.getRegisteredEndDate());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            //拼接SDK来源
            cri.where().andEquals("a.source_system", dto.getSourceSystem());
        }
        //拼接分组条件
        cri.getGroupBy().groupBy("a.game_name", "a.game_id", "a.game_classify");
        //拼接排序条件
        cri.getOrderBy().orderBy(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, dto.getSortFiled()), dto.getSortType());
        //创建sql语句 执行sql
        Sql sql = Sqls.create(gameDataTotalSql() + cri);
        //设置自定义回显对象
        sql.setCallback(Sqls.callback.entities());
        sql.setEntity(dao.getEntity(GameDataTotalVO.class));
        Pager pager = dao.createPager(dto.getPageNum(), dto.getPageSize());
        sql.setPager(pager);
        //执行sql
        dao.execute(sql);
        //得到结果集list
        List<GameDataTotalVO> list = sql.getList(GameDataTotalVO.class);
        //设置查询总数
        Sql sqlCount = Sqls.queryEntity("select count(*) from ads_game_day a " + cri);
        pager.setRecordCount((int) Daos.queryCount(dao, sqlCount));

        //根据充值时间修改值
        List<GameDataTotalVO> gameDataTotalVOList = list.stream().map(vo -> {

            //获取到一个只含有与充值时间有关的数据对象
            GameDataTotalVO tempVO = getRechargeData(vo, dto);
            //将tempVO中查出的数据添加到vo上
            copyNullProperties(tempVO, vo);

            //设置查询参数map
            Map<String, Object> dayNMap = new HashMap<>(6);
            dayNMap.put("registerBeginDate", dto.getRegisteredBeginDate());
            dayNMap.put("registerEndDate", dto.getRegisteredEndDate());
            dayNMap.put("rechargeBeginDate", dto.getRechargeBeginDate());
            dayNMap.put("rechargeEndDate", dto.getRechargeEndDate());
            dayNMap.put("gameId", vo.getGameId());
            if (StringUtils.isNotBlank(dto.getSourceSystem())) {
                dayNMap.put("sourceSystem", dto.getSourceSystem());
            }

            //买量表的充值相关数据
            DayN buyDayN = getDayNByTableName(dayNMap, "ads_dayn_game_buy");
            //自然量表的充值相关数据
            DayN natureDayN = getDayNByTableName(dayNMap, "ads_dayn_game_nature");
            //总量表的充值相关数据
            DayN dayN = getDayNByTableName(dayNMap, "ads_dayn_game");

            //设置买量数据
            //新用户充值次数、金额、人数
            vo.setBuyNewUserAmount(buyDayN == null ? BigDecimal.ZERO : buyDayN.getRechargeMoney());
            vo.setBuyNewUserAmountNum(getNewUserAmountNum(dayNMap, "buy"));
            vo.setBuyNewUserAmountCount(buyDayN == null ? 0L : buyDayN.getRechargeCount());
            //新用户复充人数
            vo.setBuyNewUserAgainNum(getNewUserAmountAgainNum(dayNMap, "buy"));

            //老用户充值次数、人数、金额
            vo.setBuyOldUserAmount(vo.getBuyAmount().subtract(vo.getBuyNewUserAmount()));
            vo.setBuyOldUserCount(vo.getBuyAmountCount() - (vo.getBuyNewUserAmountCount()));
            vo.setBuyOldUserNum(vo.getBuyAmountNum() - (vo.getBuyNewUserAmountNum()));

            //新用户付费比
            vo.setBuyNewUserRate(vo.getBuyAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getBuyNewUserAmountNum().doubleValue() / vo.getBuyAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
            //新用户付费率
            vo.setBuyNewUserRoi(vo.getBuyRegNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getBuyNewUserAmountNum().doubleValue() / vo.getBuyRegNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户客单价
            vo.setBuyNewUserAvgAmount(vo.getBuyNewUserAmountCount().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getBuyNewUserAmount().doubleValue() / vo.getBuyNewUserAmountCount().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //复充率
            vo.setBuyUserAgainRate(vo.getBuyNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getBuyNewUserAgainNum().doubleValue() / vo.getBuyNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
            //新用户复充率
            vo.setBuyNewUserAgainRate(vo.getBuyNewUserTotalAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getBuyNewUserAgainNum().doubleValue() / vo.getBuyNewUserTotalAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户付费ARPU
            vo.setBuyNewUserAmountArpu(vo.getBuyNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getBuyNewUserAmount().doubleValue() / vo.getBuyNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户回收率
            vo.setBuyNewUserRechargeRate(vo.getCost().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    vo.getBuyNewUserAmount().divide(vo.getCost(), 4, RoundingMode.HALF_UP));
            //新用户充值成本
            vo.setBuyNewUserRechargeCost(vo.getBuyNewUserAmountNum() == 0 ? BigDecimal.ZERO :
                    vo.getCost().divide(BigDecimal.valueOf(vo.getBuyNewUserAmountNum()), 4, RoundingMode.HALF_UP));

            //设置自然量数据
            //新用户充值次数、金额、人数
            vo.setNatureNewUserAmount(natureDayN == null ? BigDecimal.ZERO : natureDayN.getRechargeMoney());
            vo.setNatureNewUserAmountNum(getNewUserAmountNum(dayNMap, "nature"));
            vo.setNatureNewUserAmountCount(natureDayN == null ? 0L : natureDayN.getRechargeCount());
            //新用户复充人数
            vo.setNatureNewUserAgainNum(getNewUserAmountAgainNum(dayNMap, "nature"));

            //老用户充值次数、人数、金额
            vo.setNatureOldUserAmount(vo.getNatureAmount().subtract(vo.getNatureNewUserAmount()));
            vo.setNatureOldUserCount(vo.getNatureAmountCount() - (vo.getNatureNewUserAmountCount()));
            vo.setNatureOldUserNum(vo.getNatureAmountNum() - (vo.getNatureNewUserAmountNum()));

            //新用户付费比
            vo.setNatureNewUserRate(vo.getNatureAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNatureNewUserAmountNum().doubleValue() / vo.getNatureAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
            //新用户付费率
            vo.setNatureNewUserRoi(vo.getNatureRegNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNatureNewUserAmountNum().doubleValue() / vo.getNatureRegNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户客单价
            vo.setNatureNewUserAvgAmount(vo.getNatureNewUserAmountCount().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNatureNewUserAmount().doubleValue() / vo.getNatureNewUserAmountCount().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //复充率
            vo.setNatureUserAgainRate(vo.getNatureNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNatureNewUserAgainNum().doubleValue() / vo.getNatureNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
            //新用户复充率
            vo.setNatureNewUserAgainRate(vo.getNatureNewUserTotalAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNatureNewUserAgainNum().doubleValue() / vo.getNatureNewUserTotalAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户付费ARPU
            vo.setNatureNewUserAmountArpu(vo.getNatureNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNatureNewUserAmount().doubleValue() / vo.getNatureNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户回收率
            vo.setNatureNewUserRechargeRate(vo.getCost().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    vo.getNatureNewUserAmount().divide(vo.getCost(), 4, RoundingMode.HALF_UP));
            //新用户充值成本
            vo.setNatureNewUserRechargeCost(vo.getNatureNewUserAmountNum() == 0 ? BigDecimal.ZERO :
                    vo.getCost().divide(BigDecimal.valueOf(vo.getNatureNewUserAmountNum()), 4, RoundingMode.HALF_UP));

            //设置总量数据
            //新用户充值次数、金额、人数
            vo.setNewUserAmount(dayN == null ? BigDecimal.ZERO : dayN.getRechargeMoney());
            vo.setNewUserAmountNum(getNewUserAmountNum(dayNMap, ""));
            vo.setNewUserAmountCount(dayN == null ? 0L : dayN.getRechargeCount());
            //新用户复充人数
            vo.setNewUserAgainNum(getNewUserAmountAgainNum(dayNMap, ""));

            //老用户充值次数、人数、金额
            vo.setOldUserAmount(vo.getAmount().subtract(vo.getNewUserAmount()));
            vo.setOldUserCount(vo.getAmountCount() - (vo.getNewUserAmountCount()));
            vo.setOldUserNum(vo.getAmountNum() - (vo.getNewUserAmountNum()));

            //新用户付费比
            vo.setNewUserRate(vo.getAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNewUserAmountNum().doubleValue() / vo.getAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
            //新用户付费率
            vo.setNewUserRoi(vo.getRegNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNewUserAmountNum().doubleValue() / vo.getRegNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户客单价
            vo.setNewUserAvgAmount(vo.getNewUserAmountCount().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNewUserAmount().doubleValue() / vo.getNewUserAmountCount().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //复充率
            vo.setUserAgainRate(vo.getNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNewUserAgainNum().doubleValue() / vo.getNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
            //新用户复充率
            vo.setNewUserAgainRate(vo.getNewUserTotalAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNewUserAgainNum().doubleValue() / vo.getNewUserTotalAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户付费ARPU
            vo.setNewUserAmountArpu(vo.getNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(vo.getNewUserAmount().doubleValue() / vo.getNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

            //新用户回收率
            vo.setNewUserRechargeRate(vo.getCost().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                    vo.getNewUserAmount().divide(vo.getCost(), 4, RoundingMode.HALF_UP));
            //新用户充值成本
            vo.setNewUserRechargeCost(vo.getNewUserAmountNum() == 0 ? BigDecimal.ZERO :
                    vo.getCost().divide(BigDecimal.valueOf(vo.getNewUserAmountNum()), 4, RoundingMode.HALF_UP));

            return vo;
        }).collect(Collectors.toList());

        //返回list结果封装到page对象里
        return new Page<>(gameDataTotalVOList, pager);
    }

    /**
     * 游戏总数据总计
     *
     * @param dto 前端传递的查询参数
     * @return 返回给前端额数据
     */
    public GameDataTotalTotalVO getGameDataTotalTotal(GameDataTotalTotalDTO dto) {
        //如果注册时间参数为空，默认设置查询当天数据
        if (dto.getRegisteredBeginDate() == null || dto.getRegisteredEndDate() == null) {
            dto.setRegisteredBeginDate(LocalDate.now());
            dto.setRegisteredEndDate(LocalDate.now());
        }
        //如果充值时间参数为空，默认设置查询当天数据
        if (dto.getRechargeBeginDate() == null || dto.getRechargeEndDate() == null) {
            dto.setRechargeBeginDate(LocalDate.now());
            dto.setRechargeEndDate(LocalDate.now());
        }
        //根据传入的dto拼接查询参数
        Criteria cri = Cnd.cri();
        if (StringUtils.isNotBlank(dto.getGameName())) {
            //拼接游戏名称查询条件
            cri.where().andEquals("game_name", dto.getGameName());
        }
        if (dto.getGameId() != null) {
            //拼接游戏id查询条件
            cri.where().andEquals("game_id", dto.getGameId());
        }
        if (dto.getGameClassify() != null) {
            //拼接游戏类型查询条件
            cri.where().andEquals("game_classify", dto.getGameClassify());
        }
        if (dto.getRegisteredBeginDate() != null && dto.getRegisteredEndDate() != null) {
            //拼接注册日期查询条件
            cri.where().andBetween("dt", dto.getRegisteredBeginDate(), dto.getRegisteredEndDate());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            //拼接SDK来源
            cri.where().andEquals("source_system", dto.getSourceSystem());
        }

        Sql gameDataTotalTotalSql = Sqls.create(gameDataTotalTotalSql() + cri);
        //设置回传对象
        gameDataTotalTotalSql.setCallback(Sqls.callback.entity());
        gameDataTotalTotalSql.setEntity(dao.getEntity(GameDataTotalTotalVO.class));
        //执行sql
        dao.execute(gameDataTotalTotalSql);
        //得到查询出的结果
        GameDataTotalTotalVO vo = gameDataTotalTotalSql.getObject(GameDataTotalTotalVO.class);

        //查询充值时间相关数据
        //查询条件
        Criteria criRc = Cnd.cri();
        if (StringUtils.isNotBlank(dto.getGameName())) {
            //拼接游戏名称查询条件
            criRc.where().andEquals("game_name", dto.getGameName());
        }
        if (dto.getGameClassify() != null) {
            //拼接游戏类型查询条件
            criRc.where().andEquals("game_classify", dto.getGameClassify());
        }
        if (dto.getGameId() != null) {
            //拼接游戏id查询条件
            criRc.where().andEquals("game_id", dto.getGameId());
        }
        if (dto.getRechargeBeginDate() != null && dto.getRechargeEndDate() != null) {
            //拼接充值日期查询条件
            criRc.where().andBetween("dt", dto.getRechargeBeginDate(), dto.getRechargeEndDate());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            //拼接SDK来源
            criRc.where().andEquals("source_system", dto.getSourceSystem());
        }
        //创建sql语句
        Sql sqlWithRechargeDate = Sqls.create(gameDataTotalSqlRecharge() + criRc);
        //设置回传对象
        sqlWithRechargeDate.setCallback(Sqls.callback.entity());
        sqlWithRechargeDate.setEntity(dao.getEntity(GameDataTotalTotalVO.class));
        dao.execute(sqlWithRechargeDate);
        GameDataTotalTotalVO tempVO = sqlWithRechargeDate.getObject(GameDataTotalTotalVO.class);

        //设置查询参数map
        Map<String, Object> queryMap = new HashMap<>(4);
        queryMap.put("rechargeBeginDate", dto.getRechargeBeginDate());
        queryMap.put("rechargeEndDate", dto.getRechargeEndDate());
        queryMap.put("gameId", dto.getGameId());
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            queryMap.put("sourceSystem", dto.getSourceSystem());
        }
        //账面充值人数（总量、买量、自然量）
        Long amountNum = getNewUserAmountNum(queryMap, "");
        Long buyAmountNum = getNewUserAmountNum(queryMap, "buy");
        Long natureAmountNum = getNewUserAmountNum(queryMap, "nature");
        //设置账面充值人数
        tempVO.setAmountNum(amountNum);
        tempVO.setBuyAmountNum(buyAmountNum);
        tempVO.setNatureAmountNum(natureAmountNum);
        //设置账面ARPU(总量、买量、自然量)
        tempVO.setAmountArpu(amountNum == 0L ? BigDecimal.ZERO :
                tempVO.getAmount().divide(BigDecimal.valueOf(amountNum), 2, RoundingMode.HALF_UP));
        tempVO.setBuyAmountArpu(buyAmountNum == 0L ? BigDecimal.ZERO :
                tempVO.getBuyAmount().divide(BigDecimal.valueOf(buyAmountNum), 2, RoundingMode.HALF_UP));
        tempVO.setNatureAmountArpu(natureAmountNum == 0L ? BigDecimal.ZERO :
                tempVO.getNatureAmount().divide(BigDecimal.valueOf(natureAmountNum), 2, RoundingMode.HALF_UP));
        //将两个对象内的数据合并
        copyNullProperties(tempVO, vo);
        //获取新用户充值人数相关数据
        getGameDataTotalTotalRechargeData(dto, vo);

        return vo;
    }

    /**
     * 游戏总数据总计的新用户充值人数相关值得计算方法
     * @param dto 前端查询条件
     * @param vo 返回给前端得实体
     */
    private void getGameDataTotalTotalRechargeData(GameDataTotalTotalDTO dto, GameDataTotalTotalVO vo) {

        //设置查询参数map
        Map<String, Object> dayNMap = new HashMap<>(6);
        dayNMap.put("registerBeginDate", dto.getRegisteredBeginDate());
        dayNMap.put("registerEndDate", dto.getRegisteredEndDate());
        dayNMap.put("rechargeBeginDate", dto.getRechargeBeginDate());
        dayNMap.put("rechargeEndDate", dto.getRechargeEndDate());
        dayNMap.put("gameId", dto.getGameId());
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            dayNMap.put("sourceSystem", dto.getSourceSystem());
        }

        //买量表的充值相关数据
        DayN buyDayN = getDayNByTableName(dayNMap, "ads_dayn_game_buy");
        //自然量表的充值相关数据
        DayN natureDayN = getDayNByTableName(dayNMap, "ads_dayn_game_nature");
        //总量表的充值相关数据
        DayN dayN = getDayNByTableName(dayNMap, "ads_dayn_game");

        //设置买量数据
        //新用户充值次数、金额、人数
        vo.setBuyNewUserAmount(buyDayN == null ? BigDecimal.ZERO : buyDayN.getRechargeMoney());
        vo.setBuyNewUserAmountNum(getNewUserAmountNum(dayNMap, "buy"));
        vo.setBuyNewUserAmountCount(buyDayN == null ? 0L : buyDayN.getRechargeCount());
        //新用户复充人数
        vo.setBuyNewUserAgainNum(getNewUserAmountAgainNum(dayNMap, "buy"));

        //老用户充值次数、人数、金额
        vo.setBuyOldUserAmount(vo.getBuyAmount().subtract(vo.getBuyNewUserAmount()));
        vo.setBuyOldUserCount(vo.getBuyAmountCount() - (vo.getBuyNewUserAmountCount()));
        vo.setBuyOldUserNum(vo.getBuyAmountNum() - (vo.getBuyNewUserAmountNum()));

        //新用户付费比
        vo.setBuyNewUserRate(vo.getBuyAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getBuyNewUserAmountNum().doubleValue() / vo.getBuyAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
        //新用户付费率
        vo.setBuyNewUserRoi(vo.getBuyRegNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getBuyNewUserAmountNum().doubleValue() / vo.getBuyRegNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户客单价
        vo.setBuyNewUserAvgAmount(vo.getBuyNewUserAmountCount().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getBuyNewUserAmount().doubleValue() / vo.getBuyNewUserAmountCount().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //复充率
        vo.setBuyUserAgainRate(vo.getBuyNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getBuyNewUserAgainNum().doubleValue() / vo.getBuyNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
        //新用户复充率
        vo.setBuyNewUserAgainRate(vo.getBuyNewUserTotalAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getBuyNewUserAgainNum().doubleValue() / vo.getBuyNewUserTotalAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户付费ARPU
        vo.setBuyNewUserAmountArpu(vo.getBuyNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getBuyNewUserAmount().doubleValue() / vo.getBuyNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户回收率
        vo.setBuyNewUserRechargeRate(vo.getCost().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                vo.getBuyNewUserAmount().divide(vo.getCost(), 4, RoundingMode.HALF_UP));
        //新用户充值成本
        vo.setBuyNewUserRechargeCost(vo.getBuyNewUserAmountNum() == 0 ? BigDecimal.ZERO :
                vo.getCost().divide(BigDecimal.valueOf(vo.getBuyNewUserAmountNum()), 4, RoundingMode.HALF_UP));

        //设置自然量数据
        //新用户充值次数、金额、人数
        vo.setNatureNewUserAmount(natureDayN == null ? BigDecimal.ZERO : natureDayN.getRechargeMoney());
        vo.setNatureNewUserAmountNum(getNewUserAmountNum(dayNMap, "nature"));
        vo.setNatureNewUserAmountCount(natureDayN == null ? 0L : natureDayN.getRechargeCount());
        //新用户复充人数
        vo.setNatureNewUserAgainNum(getNewUserAmountAgainNum(dayNMap, "nature"));

        //老用户充值次数、人数、金额
        vo.setNatureOldUserAmount(vo.getNatureAmount().subtract(vo.getNatureNewUserAmount()));
        vo.setNatureOldUserCount(vo.getNatureAmountCount() - (vo.getNatureNewUserAmountCount()));
        vo.setNatureOldUserNum(vo.getNatureAmountNum() - (vo.getNatureNewUserAmountNum()));

        //新用户付费比
        vo.setNatureNewUserRate(vo.getNatureAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNatureNewUserAmountNum().doubleValue() / vo.getNatureAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
        //新用户付费率
        vo.setNatureNewUserRoi(vo.getNatureRegNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNatureNewUserAmountNum().doubleValue() / vo.getNatureRegNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户客单价
        vo.setNatureNewUserAvgAmount(vo.getNatureNewUserAmountCount().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNatureNewUserAmount().doubleValue() / vo.getNatureNewUserAmountCount().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //复充率
        vo.setNatureUserAgainRate(vo.getNatureNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNatureNewUserAgainNum().doubleValue() / vo.getNatureNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
        //新用户复充率
        vo.setNatureNewUserAgainRate(vo.getNatureNewUserTotalAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNatureNewUserAgainNum().doubleValue() / vo.getNatureNewUserTotalAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户付费ARPU
        vo.setNatureNewUserAmountArpu(vo.getNatureNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNatureNewUserAmount().doubleValue() / vo.getNatureNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户回收率
        vo.setNatureNewUserRechargeRate(vo.getCost().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                vo.getNatureNewUserAmount().divide(vo.getCost(), 4, RoundingMode.HALF_UP));
        //新用户充值成本
        vo.setNatureNewUserRechargeCost(vo.getNatureNewUserAmountNum() == 0 ? BigDecimal.ZERO :
                vo.getCost().divide(BigDecimal.valueOf(vo.getNatureNewUserAmountNum()), 4, RoundingMode.HALF_UP));

        //设置总量数据
        //新用户充值次数、金额、人数
        vo.setNewUserAmount(dayN == null ? BigDecimal.ZERO : dayN.getRechargeMoney());
        vo.setNewUserAmountNum(getNewUserAmountNum(dayNMap, ""));
        vo.setNewUserAmountCount(dayN == null ? 0L : dayN.getRechargeCount());
        //新用户复充人数
        vo.setNewUserAgainNum(getNewUserAmountAgainNum(dayNMap, ""));

        //老用户充值次数、人数、金额
        vo.setOldUserAmount(vo.getAmount().subtract(vo.getNewUserAmount()));
        vo.setOldUserCount(vo.getAmountCount() - (vo.getNewUserAmountCount()));
        vo.setOldUserNum(vo.getAmountNum() - (vo.getNewUserAmountNum()));

        //新用户付费比
        vo.setNewUserRate(vo.getAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNewUserAmountNum().doubleValue() / vo.getAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
        //新用户付费率
        vo.setNewUserRoi(vo.getRegNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNewUserAmountNum().doubleValue() / vo.getRegNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户客单价
        vo.setNewUserAvgAmount(vo.getNewUserAmountCount().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNewUserAmount().doubleValue() / vo.getNewUserAmountCount().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //复充率
        vo.setUserAgainRate(vo.getNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNewUserAgainNum().doubleValue() / vo.getNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));
        //新用户复充率
        vo.setNewUserAgainRate(vo.getNewUserTotalAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNewUserAgainNum().doubleValue() / vo.getNewUserTotalAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户付费ARPU
        vo.setNewUserAmountArpu(vo.getNewUserAmountNum().compareTo(0L) == 0 ? BigDecimal.ZERO :
                BigDecimal.valueOf(vo.getNewUserAmount().doubleValue() / vo.getNewUserAmountNum().doubleValue()).setScale(4, RoundingMode.HALF_UP));

        //新用户回收率
        vo.setNewUserRechargeRate(vo.getCost().compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                vo.getNewUserAmount().divide(vo.getCost(), 4, RoundingMode.HALF_UP));
        //新用户充值成本
        vo.setNewUserRechargeCost(vo.getNewUserAmountNum() == 0 ? BigDecimal.ZERO :
                vo.getCost().divide(BigDecimal.valueOf(vo.getNewUserAmountNum()), 4, RoundingMode.HALF_UP));

    }

    /**
     * 每日流水贡献
     *
     * @param dto 前端传递的查询参数
     * @return 返回给前端的数据
     */
    @Override
    public Page<GameDataWaterVO> getGameDataWater(GameDataWaterDTO dto) {
        if (null == dto.getRechargeDate()) {
            dto.setRechargeDate(LocalDate.now());
        }
        if (StringUtils.isBlank(dto.getSortFiled())) {
            dto.setSortFiled("amount");
        }
        if (StringUtils.isBlank(dto.getSortType())) {
            dto.setSortType(OrderByEnum.DESC.getOrderType());
        }
        Pager pager = dao.createPager(dto.getPageNum(), dto.getPageSize());
        // 创建一个 Criteria 接口实例
        SimpleCriteria cri = Cnd.cri();
        if (StringUtils.isNotBlank(dto.getGameName())) {
            cri.where().andLike("game_name", dto.getGameName());
        }
        if (StringUtils.isNotBlank(dto.getGameClassify())) {
            cri.where().andEquals("game_classify", dto.getGameClassify());
        }
        if (StringUtils.isNotBlank(dto.getGameClassify())) {
            cri.where().andEquals("game_classify", dto.getGameClassify());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            cri.where().andEquals("source_system", dto.getSourceSystem());
        }
        if (null != dto.getGameId()) {
            cri.where().andEquals("game_id", dto.getGameId());
        }
        cri.where().and("dt", "=", dto.getRechargeDate());
        cri.groupBy("source_system,game_id");
        cri.orderBy(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, dto.getSortFiled()), dto.getSortType());
        Sql sql = Sqls.queryEntity(waterSql() + "$condition");
        sql.setPager(pager);
        Entity<GameDataWaterVO> entity = dao.getEntity(GameDataWaterVO.class);
        sql.setEntity(entity).setCondition(cri);
        dao.execute(sql);

        Sql sqlCount = Sqls.queryEntity("select count(*) from ads_everyday_water " + "$condition");
        sqlCount.setCondition(cri);
        pager.setRecordCount((int) Daos.queryCount(dao, sqlCount));

        List<Long> gameIds = sql.getList(GameDataWaterVO.class).stream()
                .map(GameDataWaterVO::getGameId)
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(gameIds)) {
            return new Page<>();
        }

        SimpleCriteria templateCri = Cnd.cri();
        templateCri.where().and("dt", "=", dto.getRechargeDate());
        templateCri.where().andInList("game_id", gameIds);
        Sql templateSql = Sqls.queryEntity(waterTemplateSql() + "$condition");
        Entity<AdsEverydayWater> everydayWaterEntity = dao.getEntity(AdsEverydayWater.class);
        templateSql.setEntity(everydayWaterEntity).setCondition(templateCri);
        dao.execute(templateSql);

        Map<Long, List<AdsEverydayWater>> collect = templateSql.getList(AdsEverydayWater.class).stream().collect(Collectors.groupingBy(AdsEverydayWater::getGameId));
        List<GameDataWaterVO> result = sql.getList(GameDataWaterVO.class).stream()
                .map(item -> {
                    List<AdsEverydayWater> adsEverydayWaters = collect.get(item.getGameId());
                    if (!CollectionUtils.isEmpty(adsEverydayWaters)) {
                        List<RechargeTemplateVO> rechargeTemplateVOList = new ArrayList<>(10);
                        List<PlayerTemplateVO> playerTemplateVOS = new ArrayList<>(10);

                        adsEverydayWaters.stream().filter(amountOrder -> null != amountOrder.getAmountOrderBy()).forEach(adsEverydayWater -> {
                            if (StringUtils.isNotBlank(adsEverydayWater.getAmountOrderBy())) {
                                rechargeTemplateVOList.add(RechargeTemplateVO.builder()
                                        .amountCount(adsEverydayWater.getAmountCount())
                                        .amountNum(adsEverydayWater.getAmountNum())
                                        .amountOrderBy(Integer.valueOf(adsEverydayWater.getAmountOrderBy()))
                                        .amountPost(adsEverydayWater.getAmountPost())
                                        .build());
                            }
                        });
                        adsEverydayWaters.stream().filter(playerOrder -> null != playerOrder.getUserOrderBy()).forEach(adsEverydayWater -> {
                            if (StringUtils.isNotBlank(adsEverydayWater.getUserOrderBy())) {
                                playerTemplateVOS.add(PlayerTemplateVO.builder()
                                        .userId(adsEverydayWater.getUserId())
                                        .userAmount(adsEverydayWater.getUserAmount())
                                        .userOrderBy(Integer.valueOf(adsEverydayWater.getUserOrderBy()))
                                        .userName(adsEverydayWater.getUserName())
                                        .roleName(adsEverydayWater.getRoleName())
                                        .serverName(adsEverydayWater.getServerName())
                                        .proportion(
                                                item.getAmount().compareTo(BigDecimal.ZERO) == 0 ?
                                                        BigDecimal.ZERO :
                                                        adsEverydayWater.getUserAmount().divide(item.getAmount(), 4, RoundingMode.DOWN)
                                        )
                                        .build());
                            }
                        });
                        item.setRechargeTemplateVOList(rechargeTemplateVOList.stream().sorted(Comparator.comparing(RechargeTemplateVO::getAmountOrderBy)).collect(Collectors.toList()));
                        item.setPlayerTemplateVOS(playerTemplateVOS.stream().sorted(Comparator.comparing(PlayerTemplateVO::getUserOrderBy)).collect(Collectors.toList()));
                    }
                    return item;
                }).collect(Collectors.toList());

        return new Page<>(result, pager);
    }

    /**
     * 游戏每日复充数据
     *
     * @param dto 前端传递查询参数实体
     * @return 返回给前端的数据
     */
    @Override
    public Map<LocalDate, List<GameDataAgainDayVO>> getGameDataAgainDay(GameDataAgainDayDTO dto) {

        //如果查询时间不传递默认从今天开始查询
        if (dto.getBeginDate() == null) {
            dto.setBeginDate(LocalDate.now());
        }
        //判断开始时间后的30天是否超过当前日期
        if (dto.getBeginDate().plusDays(30).isAfter(LocalDate.now())) {
            //只查询到当前的数据
            dto.setEndDate(LocalDate.now());
        } else {
            //只考虑30天的数据
            dto.setEndDate(dto.getBeginDate().plusDays(30));
        }
        //拼接查询条件
        Criteria cri = Cnd.cri();
        if (StringUtils.isNotBlank(dto.getGameName())) {
            cri.where().andEquals("game_name", dto.getGameName());
        }
        if (dto.getGameId() != null) {
            cri.where().andEquals("game_id", dto.getGameId());
        }
        if (dto.getBeginDate() != null) {
            cri.where().andEquals("dt", dto.getBeginDate());
            cri.where().andBetween("ddf", dto.getBeginDate(), dto.getEndDate());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            //拼接SDK来源
            cri.where().andEquals("source_system", dto.getSourceSystem());
        }
        //按 "ddf" 升序排序
        cri.getOrderBy().asc("ddf");

        //获取三张表的查询结果
        List<AdsGameDayAgain> list = getListByTableType("total", cri);
        List<AdsGameDayAgainBuy> listBuySql = getListByTableType("buy", cri);
        List<AdsGameDayAgainNature> listNatureSql = getListByTableType("nature", cri);

        log.info("list长度：" + list.size());
        log.info("listBuy长度：" + listBuySql.size());
        log.info("listNature长度：" + listNatureSql.size());

        //创建Map记录数据
        Map<LocalDate, List<GameDataAgainDayVO>> map = new TreeMap<>(
                (o1, o2) -> o1.isBefore(o2) == true ? -1 : 1);
        //有序存储每个list,每个list包含当天的所有充值次数的复充数据
        List<List<GameDataAgainDayVO>> tempList = new ArrayList<>();
        //记录查询出的记录条数
        int count = list.size();
        //如果总复充list长度为0返回空结果
        if (count == 0) {
            return map;
        }
        //初始化tempList
        initList(tempList, count);
        //补全缺少的数据
        List resList = findMissDateAgainData(list, listBuySql, listNatureSql);
        List<AdsGameDayAgainBuy> listBuy = (List<AdsGameDayAgainBuy>) resList.get(0);
        List<AdsGameDayAgainNature> listNature = (List<AdsGameDayAgainNature>) resList.get(1);

        //外层循环8次 表示不同复充次数的数据
        for (int i = 0; i < 8; i++) {
            //用来计算Ci
            int ci = i + 1;
            //第一天的所有充值次数都是先计算的
            GameDataAgainDayVO tempVO = tempList.get(0).get(i);
            //设置充值次数
            tempList.get(0).get(i).setCn("C" + ci);
            //"原": Ci
            tempList.get(0).get(i).setOriginal(getCnByNum(list.get(0), ci));
            tempList.get(0).get(i).setBuyOriginal(getCnByNum(listBuy.get(0), ci));
            tempList.get(0).get(i).setNatureOriginal(getCnByNum(listNature.get(0), ci));
            //"增":初始为 0
            tempList.get(0).get(i).setIncrease(0L);
            tempList.get(0).get(i).setBuyIncrease(0L);
            tempList.get(0).get(i).setNatureIncrease(0L);
            //"移":下一个充值次数的Ci 即C(i+1)
            tempList.get(0).get(i).setDecrease(getCnByNum(list.get(0), ci + 1));
            tempList.get(0).get(i).setBuyDecrease(getCnByNum(listBuy.get(0), ci + 1));
            tempList.get(0).get(i).setNatureDecrease(getCnByNum(listNature.get(0), ci + 1));
            //"现":公式计算 = 原 + 增 - 移
            tempList.get(0).get(i).setPresent(
                    tempVO.getOriginal() + (tempVO.getIncrease()) - (tempVO.getDecrease())
            );
            tempList.get(0).get(i).setBuyPresent(
                    tempVO.getBuyOriginal() + (tempVO.getBuyIncrease()) - (tempVO.getBuyDecrease())
            );
            tempList.get(0).get(i).setNaturePresent(
                    tempVO.getNatureOriginal() + (tempVO.getNatureIncrease()) - (tempVO.getNatureDecrease())
            );
            //"比"
            tempList.get(0).get(i).setRate(getCnByNum(list.get(0), 1).compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(tempVO.getPresent().doubleValue() / getCnByNum(list.get(0), 1).doubleValue()).setScale(4, RoundingMode.HALF_UP)
            );
            tempList.get(0).get(i).setBuyRate(getCnByNum(listBuy.get(0), 1).compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(tempVO.getBuyPresent().doubleValue() / getCnByNum(listBuy.get(0), 1).doubleValue()).setScale(4, RoundingMode.HALF_UP)
            );
            tempList.get(0).get(i).setNatureRate(getCnByNum(listNature.get(0), 1).compareTo(0L) == 0 ? BigDecimal.ZERO :
                    BigDecimal.valueOf(tempVO.getNaturePresent().doubleValue() / getCnByNum(listNature.get(0), 1).doubleValue()).setScale(4, RoundingMode.HALF_UP)
            );

            //计算第2-n天的数据 复充次数i的数据
            for (int j = 1; j < count; j++) {
                //设置充值次数
                tempList.get(j).get(i).setCn("C" + ci);
                //"原":前一天的"现"
                tempList.get(j).get(i).setOriginal(tempList.get(j - 1).get(i).getPresent());
                tempList.get(j).get(i).setBuyOriginal(tempList.get(j - 1).get(i).getBuyPresent());
                tempList.get(j).get(i).setNatureOriginal(tempList.get(j - 1).get(i).getNaturePresent());
                //"现":当天的Ci - 当天的C(i+1)
                tempList.get(j).get(i).setPresent(
                        getCnByNum(list.get(j), ci) - (getCnByNum(list.get(j), ci + 1)));
                tempList.get(j).get(i).setBuyPresent(
                        getCnByNum(listBuy.get(j), ci) - (getCnByNum(listBuy.get(j), ci + 1)));
                tempList.get(j).get(i).setNaturePresent(
                        getCnByNum(listNature.get(j), ci) - (getCnByNum(listNature.get(j), ci + 1)));
                //"增":当天的Ci - 前一天的Ci
                tempList.get(j).get(i).setIncrease(
                        getCnByNum(list.get(j), ci) - (getCnByNum(list.get(j - 1), ci)));
                tempList.get(j).get(i).setBuyIncrease(
                        getCnByNum(listBuy.get(j), ci) - (getCnByNum(listBuy.get(j - 1), ci)));
                tempList.get(j).get(i).setNatureIncrease(
                        getCnByNum(listNature.get(j), ci) - (getCnByNum(listNature.get(j - 1), ci)));
                //"移":当天的C(i+1) - 前一天的C(i+1)
                tempList.get(j).get(i).setDecrease(
                        getCnByNum(list.get(j), ci + 1) - (getCnByNum(list.get(j - 1), ci + 1)));
                tempList.get(j).get(i).setBuyDecrease(
                        getCnByNum(listBuy.get(j), ci + 1) - (getCnByNum(listBuy.get(j - 1), ci + 1)));
                tempList.get(j).get(i).setNatureDecrease(
                        getCnByNum(listNature.get(j), ci + 1) - (getCnByNum(listNature.get(j - 1), ci + 1)));
                //"比":当天的现 / 前一天的C1
                tempList.get(j).get(i).setRate(getCnByNum(list.get(j - 1), 1).compareTo(0L) == 0 ? BigDecimal.ZERO :
                        BigDecimal.valueOf(tempList.get(j).get(i).getPresent().doubleValue() / getCnByNum(list.get(j - 1), 1).doubleValue()).setScale(4, RoundingMode.HALF_UP));
                tempList.get(j).get(i).setBuyRate(getCnByNum(listBuy.get(j - 1), 1).compareTo(0L) == 0 ? BigDecimal.ZERO :
                        BigDecimal.valueOf(tempList.get(j).get(i).getBuyPresent().doubleValue() / getCnByNum(listBuy.get(j - 1), 1).doubleValue()).setScale(4, RoundingMode.HALF_UP));
                tempList.get(j).get(i).setNatureRate(getCnByNum(listNature.get(j - 1), 1).compareTo(0L) == 0 ? BigDecimal.ZERO :
                        BigDecimal.valueOf(tempList.get(j).get(i).getNaturePresent().doubleValue() / getCnByNum(listNature.get(j - 1), 1).doubleValue()).setScale(4, RoundingMode.HALF_UP));
            }
        }
        //赋值时间数据
        for (int i = 0; i < tempList.size(); i++) {
            map.put(list.get(i).getDdf(), tempList.get(i));
        }

        log.info("list长度：" + list.size());
        log.info("listBuy长度：" + listBuy.size());
        log.info("listNature长度：" + listNature.size());

        //返回数据
        return map;
    }

    /**
     * 查询只与充值时间相关的数据
     *
     * @param vo  返回给前端的实体
     * @param dto 前端查询实体
     * @return GameDataTotalVO
     */
    private GameDataTotalVO getRechargeData(GameDataTotalVO vo, GameDataTotalDTO dto) {
        //查询条件
        Criteria criRc = Cnd.cri();
        if (StringUtils.isNotBlank(vo.getGameName())) {
            //拼接游戏名称查询条件
            criRc.where().andEquals("game_name", vo.getGameName());
        }
        if (vo.getGameClassify() != null) {
            //拼接游戏类型查询条件
            criRc.where().andEquals("game_classify", vo.getGameClassify());
        }
        if (vo.getGameId() != null) {
            //拼接游戏id查询条件
            criRc.where().andEquals("game_id", vo.getGameId());
        }
        if (dto.getRechargeBeginDate() != null && dto.getRechargeEndDate() != null) {
            //拼接充值日期查询条件
            criRc.where().andBetween("dt", dto.getRechargeBeginDate(), dto.getRechargeEndDate());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            //拼接SDK来源
            criRc.where().andEquals("source_system", dto.getSourceSystem());
        }
        //创建sql语句
        Sql sqlWithRechargeDate = Sqls.create(gameDataTotalSqlRecharge() + criRc);

        sqlWithRechargeDate.setCallback(Sqls.callback.entity());
        sqlWithRechargeDate.setEntity(dao.getEntity(GameDataTotalVO.class));
        dao.execute(sqlWithRechargeDate);
        //得到vo
        GameDataTotalVO gameDataTotalVO = sqlWithRechargeDate.getObject(GameDataTotalVO.class);

        //设置查询参数map
        Map<String, Object> queryMap = new HashMap<>(4);
        queryMap.put("rechargeBeginDate", dto.getRechargeBeginDate());
        queryMap.put("rechargeEndDate", dto.getRechargeEndDate());
        queryMap.put("gameId", vo.getGameId());
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            queryMap.put("sourceSystem", dto.getSourceSystem());
        }
        //账面充值人数（总量、买量、自然量）
        Long amountNum = getNewUserAmountNum(queryMap, "");
        Long buyAmountNum = getNewUserAmountNum(queryMap, "buy");
        Long natureAmountNum = getNewUserAmountNum(queryMap, "nature");
        //设置账面充值人数
        gameDataTotalVO.setAmountNum(amountNum);
        gameDataTotalVO.setBuyAmountNum(buyAmountNum);
        gameDataTotalVO.setNatureAmountNum(natureAmountNum);
        //设置账面ARPU(总量、买量、自然量)
        gameDataTotalVO.setAmountArpu(amountNum == 0L ? BigDecimal.ZERO :
                gameDataTotalVO.getAmount().divide(BigDecimal.valueOf(amountNum), 2, RoundingMode.HALF_UP));
        gameDataTotalVO.setBuyAmountArpu(buyAmountNum == 0L ? BigDecimal.ZERO :
                gameDataTotalVO.getBuyAmount().divide(BigDecimal.valueOf(buyAmountNum), 2, RoundingMode.HALF_UP));
        gameDataTotalVO.setNatureAmountArpu(natureAmountNum == 0L ? BigDecimal.ZERO :
                gameDataTotalVO.getNatureAmount().divide(BigDecimal.valueOf(natureAmountNum), 2, RoundingMode.HALF_UP));

        return gameDataTotalVO;
    }

    /**
     * 所有为空值的属性都不copy
     *
     * @param source 原数据
     * @param target 目标数据
     */
    private void copyNullProperties(Object source, Object target) {
        BeanUtils.copyProperties(source, target, getNullField(source));
    }

    /**
     * 获取属性中为空的字段
     *
     * @param target 目标对象
     * @return 不需要替换的字段数组
     */
    private static String[] getNullField(Object target) {
        BeanWrapper beanWrapper = new BeanWrapperImpl(target);
        PropertyDescriptor[] propertyDescriptors = beanWrapper.getPropertyDescriptors();
        Set<String> notNullFieldSet = new HashSet<>();
        if (propertyDescriptors.length > 0) {
            for (PropertyDescriptor p : propertyDescriptors) {
                String name = p.getName();
                Object value = beanWrapper.getPropertyValue(name);
                if (Objects.isNull(value)) {
                    notNullFieldSet.add(name);
                }
            }
        }
        String[] notNullField = new String[notNullFieldSet.size()];

        return notNullFieldSet.toArray(notNullField);
    }

    /**
     * 三张表共用查询dayN数据的方法
     *
     * @param dayNMap   参数Map
     * @param tableName 查询的表名
     * @return DayN数据
     */
    private DayN getDayNByTableName(Map<String, Object> dayNMap, String tableName) {

        //创建查询条件
        Criteria cri = Cnd.cri();
        if (dayNMap.get("registerBeginDate") != null && dayNMap.get("registerEndDate") != null) {
            //拼接注册时间查询条件
            cri.where().andBetween("dt", dayNMap.get("registerBeginDate"), dayNMap.get("registerEndDate"));
        }
        if (dayNMap.get("gameId") != null) {
            //拼接游戏ID查询条件
            cri.where().andEquals("game_id", dayNMap.get("gameId"));
        }
        if (dayNMap.get("sourceSystem") != null) {
            //拼接SDK来源查询条件
            cri.where().andEquals("source_system", dayNMap.get("sourceSystem"));
        }

        Sql dayNSql = Sqls.create("""
                SELECT
                    dayN
                FROM
                    $tableName
                """ + cri);

        //sql语句传入参数
        dayNSql.setParams(dayNMap);
        //给定查询的表名
        dayNSql.vars().set("tableName", tableName);
        //设置回传
        dayNSql.setCallback(Sqls.callback.strList());
        //执行sql 获取数据到list中
        dao.execute(dayNSql);

        //将数据库中获取的所有结果封装到一个List中
        List<String> dayNList = dayNSql.getList(String.class);
        //构建一个内部类
        DayN dayN = null;
        //解析dayNList
        if (!CollectionUtils.isEmpty(dayNList)) {
            //将dayNList中json格式数据转换为 Map
            Gson gson = new Gson();
            List<Map<String, String>> jsonList = gson.fromJson(dayNList.toString(), ArrayList.class);
            //用来存储数据的map
            Map<String, String> resMap = new HashMap<>();
            //对应的参数说明：  日期：每天充值金额-充值次数-充值人数-新用户复充人数
            resMap.put("dayN", "0.00-0-0-0");
            for (Map<String, String> jsonMap : jsonList) {
                for (Map.Entry<String, String> entry : jsonMap.entrySet()) {
                    //key是充值日期
                    String key = entry.getKey();
                    String value = entry.getValue();
                    //将value按“-”分割
                    String[] resValues = value.split("-");
                    //判断充值日期 在充值日期内的数据进行计算
                    if (DateUtil.parseLocalDate(key).compareTo((LocalDate) dayNMap.get("rechargeBeginDate")) >= 0
                            && DateUtil.parseLocalDate(key).compareTo((LocalDate) dayNMap.get("rechargeEndDate")) <= 0) {
                        //获取原先的dayN的值
                        String[] oldValues = resMap.get("dayN").split("-");
                        //充值的金额
                        BigDecimal chargeMoney = new BigDecimal(oldValues[0]).add(new BigDecimal(resValues[0]));
                        //充值的次数
                        long count = Long.parseLong(oldValues[1]) + Long.parseLong(resValues[1]);
                        //充值的人数
                        long num = Long.parseLong(oldValues[2]) + Long.parseLong(resValues[2]);
                        //新用户复充人数
                        long againNum = Long.parseLong(oldValues[3]) + Long.parseLong(resValues[3]);
                        //合并成字符串 更新dayN的值
                        String newValue = chargeMoney + "-" + count + "-" + num + "-" + againNum;
                        resMap.put("dayN", newValue);
                    }
                }
            }
            //设置dayN的参数
            if (!resMap.isEmpty() && !resMap.get("dayN").isEmpty()) {
                String[] values = resMap.get("dayN").split("-");
                dayN = DayN.builder()
                        .rechargeMoney(new BigDecimal(values[0]))
                        .rechargeCount(Long.valueOf(values[1]))
                        .build();
            }
        }

        return dayN;
    }

    /**
     * 获取新用户充值人数方法
     *
     * @param dayNMap 参数Map
     * @param type    查询的字段类型：buy、nature、总
     * @return 新用户充值人数
     */
    private Long getNewUserAmountNum(Map<String, Object> dayNMap, String type) {
        //查询新用户充值人数
        Criteria cri = Cnd.cri();
        if (dayNMap.get("registerBeginDate") != null && dayNMap.get("registerEndDate") != null) {
            //拼接注册时间查询条件
            cri.where().andBetween("reg_time", dayNMap.get("registerBeginDate"), dayNMap.get("registerEndDate"));
        }
        if (dayNMap.get("gameId") != null) {
            //拼接游戏ID查询条件
            cri.where().andEquals("game_id", dayNMap.get("gameId"));
        }
        if (dayNMap.get("sourceSystem") != null) {
            //拼接SDK来源查询条件
            cri.where().andEquals("source_system", dayNMap.get("sourceSystem"));
        }
        if (dayNMap.get("rechargeBeginDate") != null && dayNMap.get("rechargeEndDate") != null) {
            //拼接充值时间查询条件
            cri.where().andBetween("order_time", dayNMap.get("rechargeBeginDate"), dayNMap.get("rechargeEndDate"));
        }
        //拼接渠道id条件 取出自然量、买量、总量值
        if (type.equals("buy")) {
            cri.where().andNotEquals("agent_id", 0);
        } else if (type.equals("nature")) {
            cri.where().andEquals("agent_id", 0);
        }
        Sql newUserAmountNumSql = Sqls.create("""
                    SELECT
                        COUNT(DISTINCT user_id) num
                    FROM
                        game_ads.ads_information
                """ + cri);
        newUserAmountNumSql.setCallback(Sqls.callback.longValue());
        //执行sql
        dao.execute(newUserAmountNumSql);
        //得到结果 新用户充值人数
        return newUserAmountNumSql.getLong();
    }

    /**
     * 获取新用户复充人数方法
     *
     * @param dayNMap 参数列表
     * @param type    查询字段类型： buy、nature、总量
     * @return 新用户复充人数
     */
    private Long getNewUserAmountAgainNum(Map<String, Object> dayNMap, String type) {
        //查询新用户充值人数
        Criteria cri = Cnd.cri();
        if (dayNMap.get("registerBeginDate") != null && dayNMap.get("registerEndDate") != null) {
            //拼接注册时间查询条件
            cri.where().andBetween("reg_time", dayNMap.get("registerBeginDate"), dayNMap.get("registerEndDate"));
        }
        if (dayNMap.get("gameId") != null) {
            //拼接游戏ID查询条件
            cri.where().andEquals("game_id", dayNMap.get("gameId"));
        }
        if (dayNMap.get("sourceSystem") != null) {
            //拼接SDK来源查询条件
            cri.where().andEquals("source_system", dayNMap.get("sourceSystem"));
        }
        if (dayNMap.get("rechargeBeginDate") != null && dayNMap.get("rechargeEndDate") != null) {
            //拼接充值时间查询条件
            cri.where().andBetween("order_time", dayNMap.get("rechargeBeginDate"), dayNMap.get("rechargeEndDate"));
        }
        //拼接渠道id条件 取出自然量、买量、总量值
        if (type.equals("buy")) {
            cri.where().andNotEquals("agent_id", 0);
        } else if (type.equals("nature")) {
            cri.where().andEquals("agent_id", 0);
        }
        Sql newUserAmountAgainNumSql = Sqls.create("""
                    SELECT
                        COUNT(a.num)
                    FROM(
                        SELECT
                        	COUNT(user_id) num
                        FROM
                        	game_ads.ads_information
                """ + cri +
                """
                           GROUP BY user_id
                           HAVING COUNT(user_id) > 1
                           ) a;
                        """);
        newUserAmountAgainNumSql.setCallback(Sqls.callback.longValue());
        //执行sql
        dao.execute(newUserAmountAgainNumSql);
        //得到结果 新用户充值人数
        return newUserAmountAgainNumSql.getLong();
    }

    /**
     * 通过反射来获取Cn的值
     *
     * @param vo  数据库查询出来的原始数据对象实体
     * @param num 1-9
     * @return Cn
     */
    private Long getCnByNum(Object vo, int num) {
        try {

            Field f = null;
            if (vo instanceof AdsGameDayAgain) {
                f = AdsGameDayAgain.class.getDeclaredField(String.format("c%s", num));
            } else if (vo instanceof AdsGameDayAgainBuy) {
                f = AdsGameDayAgainBuy.class.getDeclaredField(String.format("buyC%s", num));
            } else if (vo instanceof AdsGameDayAgainNature) {
                f = AdsGameDayAgainNature.class.getDeclaredField(String.format("natureC%s", num));
            }
            f.setAccessible(true);
            //返回对应的Cn值
            return (Long) f.get(vo);
        } catch (Exception e) {
            throw new BaseException("get方法出错，映射出错");
        }
    }

    /**
     * 通过反射赋值每日趋势
     *
     * @param vo
     */
    private void formatDayN(GameDataDayVO vo) {
        if (CollectionUtils.isEmpty(dayNFieldMapList)) {
            return;
        }
        dayNFieldMapList.forEach(dayNFieldMap -> {
            try {
                String[] temps = ((String) dayNFieldMap.getT1().get(vo)).split("/");
                dayNFieldMap.getT2().set(vo, RechargeTrendVO.builder()
                        .rechargeMoney(new BigDecimal(temps[0]))
                        .rechargeUserCount(Long.valueOf(temps[1]))
                        .increase(new BigDecimal(temps[2]))
                        .back(new BigDecimal(temps[3]))
                        .multiples(new BigDecimal(temps[4]))
                        .build());
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        });
    }

    /**
     * 通过反射赋值每日总计趋势
     *
     * @param vo
     */
    private void formatDayNTotal(GameDataDayTotalVO vo) {
        if (CollectionUtils.isEmpty(dayNTotalFieldMapList)) {
            return;
        }
        dayNTotalFieldMapList.forEach(dayNTotalFieldMap -> {
            try {
                //得到需要计算的值
                String[] temps = ((String) dayNTotalFieldMap.getT1().get(vo)).split("/");
                //dn的金额总计
                BigDecimal dNAmount = new BigDecimal(temps[0]);
                //d1-dn的金额总计
                BigDecimal d1ToDNTotalAmount = new BigDecimal(temps[1]);
                //d1-dn的消耗总计（排除了未到时间的cost）
                BigDecimal d1ToDNTotalCost = new BigDecimal(temps[3]);
                //d1的金额总计（排除了未到时间的d1）
                BigDecimal d1Amount = new BigDecimal(temps[4]);
                //赋值
                dayNTotalFieldMap.getT2().set(vo, RechargeTrendVO.builder()
                        .rechargeMoney(dNAmount)
                        .rechargeUserCount(Long.valueOf(temps[2]))
                        .increase(d1ToDNTotalCost.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                                dNAmount.divide(d1ToDNTotalCost, 4, RoundingMode.HALF_UP))
                        .back(d1ToDNTotalCost.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                                d1ToDNTotalAmount.divide(d1ToDNTotalCost, 4, RoundingMode.HALF_UP))
                        .multiples(d1Amount.compareTo(BigDecimal.ZERO) == 0 ? BigDecimal.ZERO :
                                d1ToDNTotalAmount.divide(d1Amount, 4, RoundingMode.HALF_UP))
                        .build());

            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        });
    }

    /**
     * 初始化List数据
     *
     * @param list 需要初始化的数据
     * @param num  多少天的数据
     */
    private void initList(List<List<GameDataAgainDayVO>> list, Integer num) {
        //外层循环 num 次 表示多少天的数据
        for (int i = 0; i < num; i++) {
            list.add(new ArrayList<>());
            //内层循环8次 一共初始化8种复充次数数据对象
            for (int j = 0; j < 8; j++) {
                list.get(i).add(
                        GameDataAgainDayVO.builder()
                                .buyOriginal(0L)
                                .buyPresent(0L)
                                .buyIncrease(0L)
                                .buyDecrease(0L)
                                .buyRate(BigDecimal.ZERO)
                                .natureOriginal(0L)
                                .naturePresent(0L)
                                .natureIncrease(0L)
                                .natureDecrease(0L)
                                .natureRate(BigDecimal.ZERO)
                                .original(0L)
                                .present(0L)
                                .increase(0L)
                                .decrease(0L)
                                .rate(BigDecimal.ZERO)
                                .build()
                );
            }
        }
    }

    /**
     * 补全缺少的日期数据
     *
     * @param list       查询到的复充趋势总量原始数据
     * @param listBuy    查询到的复充趋势买量量原始数据
     * @param listNature 查询到的复充趋势自然量原始数据
     * @return 返回处理好的结果list
     */
    private List findMissDateAgainData(List<AdsGameDayAgain> list, List<AdsGameDayAgainBuy> listBuy, List<AdsGameDayAgainNature> listNature) {
        //初始化买量和自然量复充数据list
        int count = list.size();
        List<AdsGameDayAgainBuy> listBuyFinal = new ArrayList<>();
        List<AdsGameDayAgainNature> listNatureFinal = new ArrayList<>();

        //先判断三个list的数据量是否相同
        if (count == listBuy.size() && count == listNature.size()) {
            //不少数据
            List resList = new ArrayList();
            resList.add(listBuy);
            resList.add(listNature);
            return resList;
        }

        //循环赋初值
        for (int i = 0; i < count; i++) {
            listBuyFinal.add(i, AdsGameDayAgainBuy.builder()
                    .dt(list.get(i).getDt())
                    .ddf(list.get(i).getDdf())
                    .gameId(list.get(i).getGameId())
                    .gameName(list.get(i).getGameName())
                    .sourceSystem(list.get(i).getSourceSystem())
                    .buyC1(0L)
                    .buyC2(0L)
                    .buyC3(0L)
                    .buyC4(0L)
                    .buyC5(0L)
                    .buyC6(0L)
                    .buyC7(0L)
                    .buyC8(0L)
                    .buyC9(0L)
                    .build());
            listNatureFinal.add(i, AdsGameDayAgainNature.builder()
                    .dt(list.get(i).getDt())
                    .ddf(list.get(i).getDdf())
                    .gameId(list.get(i).getGameId())
                    .gameName(list.get(i).getGameName())
                    .sourceSystem(list.get(i).getSourceSystem())
                    .natureC1(0L)
                    .natureC2(0L)
                    .natureC3(0L)
                    .natureC4(0L)
                    .natureC5(0L)
                    .natureC6(0L)
                    .natureC7(0L)
                    .natureC8(0L)
                    .natureC9(0L)
                    .build());
        }

        //替换已经查询到的数据
        int tempBuy = 0;
        int tempNature = 0;
        for (int i = 0; i < count; i++) {
            if (tempBuy < listBuy.size() && listBuyFinal.get(i).getDdf().equals(listBuy.get(tempBuy).getDdf())) {
                listBuyFinal.set(i, listBuy.get(tempBuy));
                tempBuy++;
            }
            if (tempNature < listNature.size() && listNatureFinal.get(i).getDdf().equals(listNature.get(tempNature).getDdf())) {
                listNatureFinal.set(i, listNature.get(tempNature));
                tempNature++;
            }
        }
        List resList = new ArrayList();
        resList.add(listBuyFinal);
        resList.add(listNatureFinal);

        return resList;
    }

    /**
     * 不同表获取不同的list结果
     *
     * @param tableName 表明
     * @param cri       查询条件
     * @return 查询结果
     */
    private List getListByTableType(String tableName, Criteria cri) {
        //创建sql语句
        Sql againSql;
        //查询的结果封装到list中
        List list = null;
        //判断具体查询哪张表
        if (tableName.equals("total")) {
            againSql = Sqls.create(adsGameDataAgainSql() + cri);
            //自定义回传对象
            againSql.setCallback(Sqls.callback.entities());
            againSql.setEntity(dao.getEntity(AdsGameDayAgain.class));
            //执行sql
            dao.execute(againSql);
            //得到查询的原始结果
            list = againSql.getList(AdsGameDayAgain.class);
        } else if (tableName.equals("buy")) {
            againSql = Sqls.create(adsGameDataAgainBuySql() + cri);
            //自定义回传对象
            againSql.setCallback(Sqls.callback.entities());
            againSql.setEntity(dao.getEntity(AdsGameDayAgainBuy.class));
            //执行sql
            dao.execute(againSql);
            //得到查询的原始结果
            list = againSql.getList(AdsGameDayAgainBuy.class);
        } else if (tableName.equals("nature")) {
            againSql = Sqls.create(adsGameDataAgainNatureSql() + cri);
            //自定义回传对象
            againSql.setCallback(Sqls.callback.entities());
            againSql.setEntity(dao.getEntity(AdsGameDayAgainNature.class));
            //执行sql
            dao.execute(againSql);
            //得到查询的原始结果
            list = againSql.getList(AdsGameDayAgainNature.class);
        }
        //返回结果
        return list;
    }

    /**
     * 游戏每日数据SQL
     *
     * @return String
     */
    private String gameDataDaySql() {
        return """
                SELECT
                    dt cost_date,
                    game_name,
                    game_id,
                    game_classify,
                    source_system,
                    cost,
                                    
                    buy_reg_num,
                    nature_reg_num,
                    reg_num,
                        
                    buy_first_new_user_amount_count,
                    buy_first_new_user_amount_num,
                    buy_first_new_user_amount,
                    buy_old_user_count,
                    buy_old_user_num,
                    buy_old_user_amount,
                    buy_amount_count,
                    buy_amount_num,
                    buy_amount,
                    buy_new_user_total_amount_count,
                    buy_new_user_total_amount_num,
                    buy_new_user_total_amount,
                    buy_total_roi,
                    buy_first_roi,
                    buy_first_amount_rate,
                    buy_today_amount_rate,
                    buy_new_user_rate,
                    buy_first_avg_amount,
                    buy_today_avg_amount,
                    buy_avg_amount,
                    buy_user_again_rate,
                    buy_reg_user_arpu,
                    buy_first_amount_arpu,
                    buy_today_amount_arpu,
                    buy_amount_arpu,
                    buy_amount_d1,
                    buy_amount_d2,
                    buy_amount_d3,
                    buy_amount_d4,
                    buy_amount_d5,
                    buy_amount_d6,
                    buy_amount_d7,
                    buy_amount_d8,
                    buy_amount_d9,
                    buy_amount_d10,
                    buy_amount_d11,
                    buy_amount_d12,
                    buy_amount_d13,
                    buy_amount_d14,
                    buy_amount_d15,
                    buy_amount_d16,
                    buy_amount_d17,
                    buy_amount_d18,
                    buy_amount_d19,
                    buy_amount_d20,
                    buy_amount_d21,
                    buy_amount_d22,
                    buy_amount_d23,
                    buy_amount_d24,
                    buy_amount_d25,
                    buy_amount_d26,
                    buy_amount_d27,
                    buy_amount_d28,
                    buy_amount_d29,
                    buy_amount_m1,
                    buy_amount_m2,
                    buy_amount_m3,
                    buy_amount_m6,
                    buy_amount_y1,
                    buy_amount_sum,
                    buy_reg_cost,
                    buy_first_amount_cost as buy_first_new_user_recharge_cost,
                    buy_total_amount_cost as buy_total_recharge_cost,
                    buy_hundred_user_num,
                    buy_hundred_user_num_cost,
                    buy_first_role_num,
                    buy_role_num,
                    buy_new_user_total_role_num,
                    round(IF(buy_first_role_num > 0, cost / buy_first_role_num, 0), 2) buy_first_role_num_cost,
                    round(IF(buy_role_num > 0, cost / buy_role_num, 0), 2) buy_role_num_cost,
                    round(IF(buy_new_user_total_role_num >0, cost / buy_new_user_total_role_num, 0), 2) buy_new_user_total_role_num_cost,
                    round(IF(buy_reg_num >0, buy_first_role_num / buy_reg_num, 0), 4) buy_first_role_num_rate,
                    round(IF(buy_reg_num >0, buy_role_num / buy_reg_num, 0), 4) buy_role_num_rate,
                    round(IF(buy_reg_num >0, buy_new_user_total_role_num / buy_reg_num, 0), 4) buy_new_user_total_role_num_rate,
                        
                    nature_first_new_user_amount_count,
                    nature_first_new_user_amount_num,
                    nature_first_new_user_amount,
                    nature_old_user_count,
                    nature_old_user_num,
                    nature_old_user_amount,
                    nature_amount_count,
                    nature_amount_num,
                    nature_amount,
                    nature_new_user_total_amount_count,
                    nature_new_user_total_amount_num,
                    nature_new_user_total_amount,
                    nature_total_roi,
                    nature_first_roi,
                    nature_first_amount_rate,
                    nature_today_amount_rate,
                    nature_new_user_rate,
                    nature_first_avg_amount,
                    nature_today_avg_amount,
                    nature_avg_amount,
                    nature_user_again_rate,
                    nature_reg_user_arpu,
                    nature_first_amount_arpu,
                    nature_today_amount_arpu,
                    nature_amount_arpu,
                    nature_amount_d1,
                    nature_amount_d2,
                    nature_amount_d3,
                    nature_amount_d4,
                    nature_amount_d5,
                    nature_amount_d6,
                    nature_amount_d7,
                    nature_amount_d8,
                    nature_amount_d9,
                    nature_amount_d10,
                    nature_amount_d11,
                    nature_amount_d12,
                    nature_amount_d13,
                    nature_amount_d14,
                    nature_amount_d15,
                    nature_amount_d16,
                    nature_amount_d17,
                    nature_amount_d18,
                    nature_amount_d19,
                    nature_amount_d20,
                    nature_amount_d21,
                    nature_amount_d22,
                    nature_amount_d23,
                    nature_amount_d24,
                    nature_amount_d25,
                    nature_amount_d26,
                    nature_amount_d27,
                    nature_amount_d28,
                    nature_amount_d29,
                    nature_amount_m1,
                    nature_amount_m2,
                    nature_amount_m3,
                    nature_amount_m6,
                    nature_amount_y1,
                    nature_amount_sum,
                    nature_reg_cost,
                    nature_first_amount_cost as nature_first_new_user_recharge_cost,
                    nature_total_amount_cost as nature_total_recharge_cost,
                    nature_hundred_user_num,
                    nature_hundred_user_num_cost,
                    nature_first_role_num,
                    nature_role_num,
                    nature_new_user_total_role_num,
                    round(IF(nature_first_role_num > 0, cost / nature_first_role_num, 0), 2) nature_first_role_num_cost,
                    round(IF(nature_role_num > 0, cost / nature_role_num, 0), 2) nature_role_num_cost,
                    round(IF(nature_new_user_total_role_num >0, cost / nature_new_user_total_role_num, 0), 2) nature_new_user_total_role_num_cost,
                    round(IF(nature_reg_num >0, nature_first_role_num / nature_reg_num, 0), 4) nature_first_role_num_rate,
                    round(IF(nature_reg_num >0, nature_role_num / nature_reg_num, 0), 4) nature_role_num_rate,
                    round(IF(nature_reg_num >0, nature_new_user_total_role_num / nature_reg_num, 0), 4) nature_new_user_total_role_num_rate,
                        
                    first_new_user_amount_count,
                    first_new_user_amount_num,
                    first_new_user_amount,
                    old_user_count,
                    old_user_num,
                    old_user_amount,
                    amount_count,
                    amount_num,
                    amount,
                    new_user_total_amount_count,
                    new_user_total_amount_num,
                    new_user_total_amount,
                    total_roi,
                    first_roi,
                    first_amount_rate,
                    today_amount_rate,
                    new_user_rate,
                    first_avg_amount,
                    today_avg_amount,
                    avg_amount,
                    user_again_rate,
                    reg_user_arpu,
                    first_amount_arpu,
                    today_amount_arpu,
                    amount_arpu,
                    amount_d1,
                    amount_d2,
                    amount_d3,
                    amount_d4,
                    amount_d5,
                    amount_d6,
                    amount_d7,
                    amount_d8,
                    amount_d9,
                    amount_d10,
                    amount_d11,
                    amount_d12,
                    amount_d13,
                    amount_d14,
                    amount_d15,
                    amount_d16,
                    amount_d17,
                    amount_d18,
                    amount_d19,
                    amount_d20,
                    amount_d21,
                    amount_d22,
                    amount_d23,
                    amount_d24,
                    amount_d25,
                    amount_d26,
                    amount_d27,
                    amount_d28,
                    amount_d29,
                    amount_m1,
                    amount_m2,
                    amount_m3,
                    amount_m6,
                    amount_y1,
                    amount_sum,
                    reg_cost,
                    first_amount_cost as first_new_user_recharge_cost,
                    total_amount_cost as total_recharge_cost,
                    hundred_user_num,
                    hundred_user_num_cost,
                    first_role_num,
                    role_num,
                    new_user_total_role_num,
                    round(IF(first_role_num > 0, cost / first_role_num, 0), 2) first_role_num_cost,
                    round(IF(role_num > 0, cost / role_num, 0), 2) role_num_cost,
                    round(IF(new_user_total_role_num >0, cost / new_user_total_role_num, 0), 2) new_user_total_role_num_cost,
                    round(IF(reg_num >0, first_role_num / reg_num, 0), 4) first_role_num_rate,
                    round(IF(reg_num >0, role_num / reg_num, 0), 4) role_num_rate,
                    round(IF(reg_num >0, new_user_total_role_num / reg_num, 0), 4) new_user_total_role_num_rate
                    
                FROM
                    ads_game_day
                """;
    }

    /**
     * 游戏每日数据总计sql
     *
     * @return String
     */
    private String gameDataDayTotalSql() {
        return """
                SELECT
                IFNULL(SUM(cost), 0) cost,
                                
                IFNULL(SUM(buy_reg_num), 0) buy_reg_num,
                IFNULL(SUM(nature_reg_num), 0) nature_reg_num,
                IFNULL(SUM(reg_num), 0) reg_num,
                                
                """
                + amountDay("buy_") +
                """
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now() , CAST(SPLIT_PART(buy_amount_m1 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_m1, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_m1, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now(), CAST(SPLIT_PART(buy_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS buy_amount_m1,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now() , CAST(SPLIT_PART(buy_amount_m2 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_m2, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_m2, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now(), CAST(SPLIT_PART(buy_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS buy_amount_m2,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now() , CAST(SPLIT_PART(buy_amount_m3 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_m3, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_m3, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now(), CAST(SPLIT_PART(buy_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS buy_amount_m3,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now() , CAST(SPLIT_PART(buy_amount_m6 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_m6, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_m6, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now(), CAST(SPLIT_PART(buy_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS buy_amount_m6,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now() , CAST(SPLIT_PART(buy_amount_y1 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_y1, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_y1, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now(), CAST(SPLIT_PART(buy_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS buy_amount_y1,
                        CONCAT(
                        	SUM(CAST(SPLIT_PART(buy_amount_sum , '/', 1) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_sum, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_sum, '/', 2) AS BIGINT)), '/',
                            SUM(cost), '/',
                            SUM(CAST(SPLIT_PART(buy_amount_d1 , '/', 1) AS DECIMAL(10, 2)))
                            ) AS buy_amount_sum,
                        IFNULL(SUM(buy_first_new_user_amount_count), 0) buy_first_new_user_amount_count,
                        IFNULL(SUM(buy_first_new_user_amount_num), 0) buy_first_new_user_amount_num,
                        IFNULL(SUM(buy_first_new_user_amount), 0) buy_first_new_user_amount,
                        IFNULL(SUM(buy_old_user_count), 0) buy_old_user_count,
                        IFNULL(SUM(buy_old_user_num), 0) buy_old_user_num,
                        IFNULL(SUM(buy_old_user_amount), 0) buy_old_user_amount,
                        IFNULL(SUM(buy_amount_count), 0) buy_amount_count,
                        IFNULL(SUM(buy_amount_num), 0) buy_amount_num,
                        IFNULL(SUM(buy_amount), 0) buy_amount,
                        IFNULL(SUM(buy_new_user_total_amount_count), 0) buy_new_user_total_amount_count,
                        IFNULL(SUM(buy_new_user_total_amount_num), 0) buy_new_user_total_amount_num,
                        IFNULL(SUM(buy_new_user_total_amount), 0) buy_new_user_total_amount,
                        round(if(SUM(cost) > 0 , SUM(buy_first_new_user_amount) / SUM(cost) ,0), 4) buy_first_roi,
                        round(if(SUM(buy_reg_num) > 0 , SUM(buy_first_new_user_amount_num) / SUM(buy_reg_num) ,0), 4) buy_first_amount_rate,
                        round(if(SUM(buy_reg_num) > 0, SUM(buy_new_user_total_amount_num) / SUM(buy_reg_num), 0) ,4) buy_today_amount_rate,
                        round(if(SUM(buy_amount_num) > 0 , SUM(buy_first_new_user_amount_num) / SUM(buy_amount_num) ,0), 4) buy_new_user_rate,
                        round(if(SUM(buy_first_new_user_amount_count) > 0, SUM(buy_first_new_user_amount) / SUM(buy_first_new_user_amount_count), 0), 2) buy_first_avg_amount,
                        round(if(SUM(buy_new_user_total_amount_count) > 0, SUM(buy_new_user_total_amount) / SUM(buy_new_user_total_amount_count), 0), 2) buy_today_avg_amount,
                        round(if(SUM(buy_amount_count) > 0, SUM(buy_amount) / SUM(buy_amount_count), 0), 2) buy_avg_amount,
                        round(if(SUM(buy_new_user_total_amount_num) > 0, SUM(buy_reg_order_user_again) / SUM(buy_new_user_total_amount_num), 0), 4) buy_user_again_rate,
                        round(if(SUM(buy_reg_num) > 0, SUM(buy_new_user_total_amount) / SUM(buy_reg_num), 0), 2) buy_reg_user_arpu,
                        round(if(SUM(buy_first_new_user_amount_num) > 0 , SUM(buy_first_new_user_amount) / SUM(buy_first_new_user_amount_num), 0), 2) buy_first_amount_arpu,
                        round(if(SUM(buy_new_user_total_amount_num) > 0 , SUM(buy_new_user_total_amount) / SUM(buy_new_user_total_amount_num), 0), 2) buy_today_amount_arpu,
                        round(if(SUM(buy_amount_num) > 0, SUM(buy_amount) / SUM(buy_amount_num), 0), 2) buy_amount_arpu,
                        round(if(SUM(buy_reg_num) > 0, SUM(cost) / SUM(buy_reg_num), 0), 2) buy_reg_cost,
                        round(if(SUM(buy_first_new_user_amount_num) > 0, SUM(cost) / SUM(buy_first_new_user_amount_num), 0), 2) buy_first_new_user_recharge_cost,
                        round(if(SUM(buy_new_user_total_amount_num) > 0, SUM(cost) / SUM(buy_new_user_total_amount_num), 0), 2) buy_total_recharge_cost,
                        round(if(SUM(cost) > 0, SUM(buy_new_user_total_amount) / SUM(cost), 0), 4) buy_total_roi,
                        SUM(buy_hundred_user_num) buy_hundred_user_num,
                        round(IF(SUM(buy_hundred_user_num) > 0, SUM(cost) / SUM(buy_hundred_user_num), 0), 2) buy_hundred_user_num_cost,
                        SUM(buy_first_role_num) buy_first_role_num,
                        SUM(buy_role_num) buy_role_num,
                        SUM(buy_new_user_total_role_num) buy_new_user_total_role_num,
                        round(IF(SUM(buy_first_role_num) > 0, SUM(cost) / SUM(buy_first_role_num), 0), 2) buy_first_role_num_cost,
                        round(IF(SUM(buy_role_num) > 0, SUM(cost) / SUM(buy_role_num), 0), 2) buy_role_num_cost,
                        round(IF(SUM(buy_new_user_total_role_num) >0, SUM(cost) / SUM(buy_new_user_total_role_num), 0), 2) buy_new_user_total_role_num_cost,
                        round(IF(SUM(buy_reg_num) >0, SUM(buy_first_role_num) / SUM(buy_reg_num), 0), 4) buy_first_role_num_rate,
                        round(IF(SUM(buy_reg_num) >0, SUM(buy_role_num) / SUM(buy_reg_num), 0), 4) buy_role_num_rate,
                        round(IF(SUM(buy_reg_num) >0, SUM(buy_new_user_total_role_num) / SUM(buy_reg_num), 0), 4) buy_new_user_total_role_num_rate,
                                        
                        """
                + amountDay("nature_") +
                """
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now() , CAST(SPLIT_PART(nature_amount_m1 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_m1, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_m1, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now(), CAST(SPLIT_PART(nature_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS nature_amount_m1,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now() , CAST(SPLIT_PART(nature_amount_m2 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_m2, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_m2, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now(), CAST(SPLIT_PART(nature_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS nature_amount_m2,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now() , CAST(SPLIT_PART(nature_amount_m3 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_m3, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_m3, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now(), CAST(SPLIT_PART(nature_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS nature_amount_m3,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now() , CAST(SPLIT_PART(nature_amount_m6 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_m6, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_m6, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now(), CAST(SPLIT_PART(nature_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS nature_amount_m6,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now() , CAST(SPLIT_PART(nature_amount_y1 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_y1, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_y1, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now(), CAST(SPLIT_PART(nature_amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS nature_amount_y1,
                        CONCAT(
                        	SUM(CAST(SPLIT_PART(nature_amount_sum , '/', 1) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_sum, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_sum, '/', 2) AS BIGINT)), '/',
                            SUM(cost), '/',
                            SUM(CAST(SPLIT_PART(nature_amount_d1 , '/', 1) AS DECIMAL(10, 2)))
                            ) AS nature_amount_sum,
                        IFNULL(SUM(nature_first_new_user_amount_count), 0) nature_first_new_user_amount_count,
                        IFNULL(SUM(nature_first_new_user_amount_num), 0) nature_first_new_user_amount_num,
                        IFNULL(SUM(nature_first_new_user_amount), 0) nature_first_new_user_amount,
                        IFNULL(SUM(nature_old_user_count), 0) nature_old_user_count,
                        IFNULL(SUM(nature_old_user_num), 0) nature_old_user_num,
                        IFNULL(SUM(nature_old_user_amount), 0) nature_old_user_amount,
                        IFNULL(SUM(nature_amount_count), 0) nature_amount_count,
                        IFNULL(SUM(nature_amount_num), 0) nature_amount_num,
                        IFNULL(SUM(nature_amount), 0) nature_amount,
                        IFNULL(SUM(nature_new_user_total_amount_count), 0) nature_new_user_total_amount_count,
                        IFNULL(SUM(nature_new_user_total_amount_num), 0) nature_new_user_total_amount_num,
                        IFNULL(SUM(nature_new_user_total_amount), 0) nature_new_user_total_amount,
                        round(if(SUM(cost) > 0 , SUM(nature_first_new_user_amount) / SUM(cost) ,0), 4) nature_first_roi,
                        round(if(SUM(nature_reg_num) > 0 , SUM(nature_first_new_user_amount_num) / SUM(nature_reg_num) ,0), 4) nature_first_amount_rate,
                        round(if(SUM(nature_reg_num) > 0, SUM(nature_new_user_total_amount_num) / SUM(nature_reg_num), 0) ,4) nature_today_amount_rate,
                        round(if(SUM(nature_amount_num) > 0 , SUM(nature_first_new_user_amount_num) / SUM(nature_amount_num) ,0), 4) nature_new_user_rate,
                        round(if(SUM(nature_first_new_user_amount_count) > 0, SUM(nature_first_new_user_amount) / SUM(nature_first_new_user_amount_count), 0), 2) nature_first_avg_amount,
                        round(if(SUM(nature_new_user_total_amount_count) > 0, SUM(nature_new_user_total_amount) / SUM(nature_new_user_total_amount_count), 0), 2) nature_today_avg_amount,
                        round(if(SUM(nature_amount_count) > 0, SUM(nature_amount) / SUM(nature_amount_count), 0), 2) nature_avg_amount,
                        round(if(SUM(nature_new_user_total_amount_num) > 0, SUM(nature_reg_order_user_again) / SUM(nature_new_user_total_amount_num), 0), 4) nature_user_again_rate,
                        round(if(SUM(nature_reg_num) > 0, SUM(nature_new_user_total_amount) / SUM(nature_reg_num), 0), 2) nature_reg_user_arpu,
                        round(if(SUM(nature_first_new_user_amount_num) > 0 , SUM(nature_first_new_user_amount) / SUM(nature_first_new_user_amount_num), 0), 2) nature_first_amount_arpu,
                        round(if(SUM(nature_new_user_total_amount_num) > 0 , SUM(nature_new_user_total_amount) / SUM(nature_new_user_total_amount_num), 0), 2) nature_today_amount_arpu,
                        round(if(SUM(nature_amount_num) > 0, SUM(nature_amount) / SUM(nature_amount_num), 0), 2) nature_amount_arpu,
                        round(if(SUM(nature_reg_num) > 0, SUM(cost) / SUM(nature_reg_num), 0), 2) nature_reg_cost,
                        round(if(SUM(nature_first_new_user_amount_num) > 0, SUM(cost) / SUM(nature_first_new_user_amount_num), 0), 2) nature_first_new_user_recharge_cost,
                        round(if(SUM(nature_new_user_total_amount_num) > 0, SUM(cost) / SUM(nature_new_user_total_amount_num), 0), 2) nature_total_recharge_cost,
                        round(if(SUM(cost) > 0, SUM(nature_new_user_total_amount) / SUM(cost), 0), 4) nature_total_roi,
                        SUM(nature_hundred_user_num) nature_hundred_user_num,
                        round(IF(SUM(nature_hundred_user_num) > 0, SUM(cost) / SUM(nature_hundred_user_num), 0), 2) nature_hundred_user_num_cost,
                        SUM(nature_first_role_num) nature_first_role_num,
                        SUM(nature_role_num) nature_role_num,
                        SUM(nature_new_user_total_role_num) nature_new_user_total_role_num,
                        round(IF(SUM(nature_first_role_num) > 0, SUM(cost) / SUM(nature_first_role_num), 0), 2) nature_first_role_num_cost,
                        round(IF(SUM(nature_role_num) > 0, SUM(cost) / SUM(nature_role_num), 0), 2) nature_role_num_cost,
                        round(IF(SUM(nature_new_user_total_role_num) >0, SUM(cost) / SUM(nature_new_user_total_role_num), 0), 2) nature_new_user_total_role_num_cost,
                        round(IF(SUM(nature_reg_num) >0, SUM(nature_first_role_num) / SUM(nature_reg_num), 0), 4) nature_first_role_num_rate,
                        round(IF(SUM(nature_reg_num) >0, SUM(nature_role_num) / SUM(nature_reg_num), 0), 4) nature_role_num_rate,
                        round(IF(SUM(nature_reg_num) >0, SUM(nature_new_user_total_role_num) / SUM(nature_reg_num), 0), 4) nature_new_user_total_role_num_rate,
                                        
                        """
                + amountDay("") +
                """
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now() , CAST(SPLIT_PART(amount_m1 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(amount_m1, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(amount_m1, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 29 day) <= LocalDate.now(), CAST(SPLIT_PART(amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS amount_m1,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now() , CAST(SPLIT_PART(amount_m2 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(amount_m2, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(amount_m2, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 59 day) <= LocalDate.now(), CAST(SPLIT_PART(amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS amount_m2,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now() , CAST(SPLIT_PART(amount_m3 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(amount_m3, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(amount_m3, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 89 day) <= LocalDate.now(), CAST(SPLIT_PART(amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS amount_m3,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now() , CAST(SPLIT_PART(amount_m6 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(amount_m6, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(amount_m6, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 179 day) <= LocalDate.now(), CAST(SPLIT_PART(amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS amount_m6,
                        CONCAT(
                        	SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now() , CAST(SPLIT_PART(amount_y1 , '/', 1) AS DECIMAL(10, 2)), 0 )), '/',
                            SUM(CAST(SPLIT_PART(amount_y1, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(amount_y1, '/', 2) AS BIGINT)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now(), cost, 0)), '/',
                            SUM(IF( DATE_ADD(dt, INTERVAL 1 year) <= LocalDate.now(), CAST(SPLIT_PART(amount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                            ) AS amount_y1,
                        CONCAT(
                        	SUM(CAST(SPLIT_PART(amount_sum , '/', 1) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(amount_sum, '/', 6) AS DECIMAL(10, 2))), '/',
                            SUM(CAST(SPLIT_PART(amount_sum, '/', 2) AS BIGINT)), '/',
                            SUM(cost), '/',
                            SUM(CAST(SPLIT_PART(amount_d1 , '/', 1) AS DECIMAL(10, 2)))
                            ) AS amount_sum,
                        IFNULL(SUM(first_new_user_amount_count), 0) first_new_user_amount_count,
                        IFNULL(SUM(first_new_user_amount_num), 0) first_new_user_amount_num,
                        IFNULL(SUM(first_new_user_amount), 0) first_new_user_amount,
                        IFNULL(SUM(old_user_count), 0) old_user_count,
                        IFNULL(SUM(old_user_num), 0) old_user_num,
                        IFNULL(SUM(old_user_amount), 0) old_user_amount,
                        IFNULL(SUM(amount_count), 0) amount_count,
                        IFNULL(SUM(amount_num), 0) amount_num,
                        IFNULL(SUM(amount), 0) amount,
                        IFNULL(SUM(new_user_total_amount_count), 0) new_user_total_amount_count,
                        IFNULL(SUM(new_user_total_amount_num), 0) new_user_total_amount_num,
                        IFNULL(SUM(new_user_total_amount), 0) new_user_total_amount,
                        round(if(SUM(cost) > 0 , SUM(first_new_user_amount) / SUM(cost) ,0), 4) first_roi,
                        round(if(SUM(reg_num) > 0 , SUM(first_new_user_amount_num) / SUM(reg_num) ,0), 4) first_amount_rate,
                        round(if(SUM(reg_num) > 0, SUM(new_user_total_amount_num) / SUM(reg_num), 0) ,4) today_amount_rate,
                        round(if(SUM(amount_num) > 0 , SUM(first_new_user_amount_num) / SUM(amount_num) ,0), 4) new_user_rate,
                        round(if(SUM(first_new_user_amount_count) > 0, SUM(first_new_user_amount) / SUM(first_new_user_amount_count), 0), 2) first_avg_amount,
                        round(if(SUM(new_user_total_amount_count) > 0, SUM(new_user_total_amount) / SUM(new_user_total_amount_count), 0), 2) today_avg_amount,
                        round(if(SUM(amount_count) > 0, SUM(amount) / SUM(amount_count), 0), 2) avg_amount,
                        round(if(SUM(new_user_total_amount_num) > 0, SUM(reg_order_user_again) / SUM(new_user_total_amount_num), 0), 4) user_again_rate,
                        round(if(SUM(reg_num) > 0, SUM(new_user_total_amount) / SUM(reg_num), 0), 2) reg_user_arpu,
                        round(if(SUM(first_new_user_amount_num) > 0 , SUM(first_new_user_amount) / SUM(first_new_user_amount_num), 0), 2) first_amount_arpu,
                        round(if(SUM(new_user_total_amount_num) > 0 , SUM(new_user_total_amount) / SUM(new_user_total_amount_num), 0), 2) today_amount_arpu,
                        round(if(SUM(amount_num) > 0, SUM(amount) / SUM(amount_num), 0), 2) amount_arpu,
                        round(if(SUM(reg_num) > 0, SUM(cost) / SUM(reg_num), 0), 2) reg_cost,
                        round(if(SUM(first_new_user_amount_num) > 0, SUM(cost) / SUM(first_new_user_amount_num), 0), 2) first_new_user_recharge_cost,
                        round(if(SUM(new_user_total_amount_num) > 0, SUM(cost) / SUM(new_user_total_amount_num), 0), 2) total_recharge_cost,
                        round(if(SUM(cost) > 0, SUM(new_user_total_amount) / SUM(cost), 0), 4) total_roi,
                        SUM(hundred_user_num) hundred_user_num,
                        round(IF(SUM(hundred_user_num) > 0, SUM(cost) / SUM(hundred_user_num), 0), 2) hundred_user_num_cost,
                        SUM(first_role_num) first_role_num,
                        SUM(role_num) role_num,
                        SUM(new_user_total_role_num) new_user_total_role_num,
                        round(IF(SUM(first_role_num) > 0, SUM(cost) / SUM(first_role_num), 0), 2) first_role_num_cost,
                        round(IF(SUM(role_num) > 0, SUM(cost) / SUM(role_num), 0), 2) role_num_cost,
                        round(IF(SUM(new_user_total_role_num) >0, SUM(cost) / SUM(new_user_total_role_num), 0), 2) new_user_total_role_num_cost,
                        round(IF(SUM(reg_num) >0, SUM(first_role_num) / SUM(reg_num), 0), 4) first_role_num_rate,
                        round(IF(SUM(reg_num) >0, SUM(role_num) / SUM(reg_num), 0), 4) role_num_rate,
                        round(IF(SUM(reg_num) >0, SUM(new_user_total_role_num) / SUM(reg_num), 0), 4) new_user_total_role_num_rate
                                        
                        FROM
                        ads_game_day
                         
                                        
                        """;
    }

    /**
     * 游戏每日总计趋势sql
     *
     * @param type 输入的参数表示选择表的类型
     * @return String : Dn的充值金额 / D1-Dn的充值总金额 / Dn的充值人数 /当前消耗(剔除不存在的天数数据) /D1充值金额总和(剔除不存在的天数数据)/type
     */
    private String amountDay(String type) {
        //拼接查询条件
        StringBuilder trendDay = new StringBuilder(StringUtils.EMPTY);
        for (int day = 1; day <= 29; day++) {
            trendDay.append("""
                    CONCAT(
                    SUM(IF( DATE_ADD(dt, INTERVAL %s day) <= LocalDate.now()  , CAST(SPLIT_PART(%samount_d%s , '/', 1) AS DECIMAL(10, 2)), 0 )),'/',
                    SUM(CAST(SPLIT_PART( %samount_d%s, '/', 6) AS DECIMAL(10, 2))),'/',
                    SUM(CAST(SPLIT_PART(%samount_d%s, '/', 2) AS BIGINT)),'/',
                    SUM(IF( DATE_ADD(dt, INTERVAL %s day) <= LocalDate.now(),  cost, 0)),'/',
                    SUM(IF( DATE_ADD(dt, INTERVAL %s day) <= LocalDate.now(),  CAST(SPLIT_PART(%samount_d1 , '/', 1) AS DECIMAL(10, 2)) , 0))
                    ) AS %samount_d%s ,
                       """.formatted(day - 1, type, day, type, day, type, day, day - 1, day - 1, type, type, day));
        }

        return trendDay.toString();
    }

    /**
     * 游戏总数据SQL
     *
     * @return String
     */
    private String gameDataTotalSql() {
        return """
                SELECT
                    a.game_id game_id,
                    a.game_name game_name,
                    a.game_classify game_classify,
                    
                    sum(a.buy_reg_num) buy_reg_num,
                    sum(a.nature_reg_num) nature_reg_num,
                    sum(a.reg_num) reg_num,
                    sum(a.cost) cost,
                    
                    sum(a.buy_first_new_user_amount_count) buy_first_new_user_amount_count,
                    sum(a.buy_first_new_user_amount_num) buy_first_new_user_amount_num,
                    sum(a.buy_first_new_user_amount) buy_first_new_user_amount,
                    sum(a.nature_first_new_user_amount_count) nature_first_new_user_amount_count,
                    sum(a.nature_first_new_user_amount_num) nature_first_new_user_amount_num,
                    sum(a.nature_first_new_user_amount) nature_first_new_user_amount,
                    sum(a.first_new_user_amount_count) first_new_user_amount_count,
                    sum(a.first_new_user_amount_num) first_new_user_amount_num,
                    sum(a.first_new_user_amount) first_new_user_amount,
                    
                    sum(a.buy_new_user_total_amount_count) buy_new_user_total_amount_count,
                    sum(a.buy_new_user_total_amount_num) buy_new_user_total_amount_num,
                    sum(a.buy_new_user_total_amount) buy_new_user_total_amount,
                    sum(a.nature_new_user_total_amount_count) nature_new_user_total_amount_count,
                    sum(a.nature_new_user_total_amount_num) nature_new_user_total_amount_num,
                    sum(a.nature_new_user_total_amount) nature_new_user_total_amount,
                    sum(a.new_user_total_amount_count) new_user_total_amount_count,
                    sum(a.new_user_total_amount_num) new_user_total_amount_num,
                    sum(a.new_user_total_amount) new_user_total_amount,
                    
                    round(if(SUM(a.cost) > 0 , SUM(buy_first_new_user_amount) / SUM(a.cost) ,0), 4) buy_first_roi,
                    round(if(sum(a.buy_reg_num) > 0 ,sum(a.buy_first_new_user_amount_num) / sum(a.buy_reg_num), 0), 4) buy_first_amount_rate,
                    round(if(sum(a.buy_reg_num) > 0, sum(a.buy_new_user_total_amount_num) / sum(a.buy_reg_num), 0), 4) buy_today_amount_rate,
                    round(if(SUM(a.cost) > 0 , SUM(a.nature_first_new_user_amount) / SUM(a.cost) ,0), 4) nature_first_roi,
                    round(if(sum(a.nature_reg_num) > 0 ,sum(a.nature_first_new_user_amount_num) / sum(a.nature_reg_num), 0), 4) nature_first_amount_rate,
                    round(if(sum(a.nature_reg_num) > 0, sum(a.nature_new_user_total_amount_num) / sum(a.nature_reg_num), 0), 4) nature_today_amount_rate,
                    round(if(SUM(a.cost) > 0 , SUM(first_new_user_amount) / SUM(a.cost) ,0), 4) first_roi,
                    round(if(sum(a.reg_num) > 0 ,sum(a.first_new_user_amount_num) / sum(a.reg_num), 0), 4) first_amount_rate,
                    round(if(sum(a.reg_num) > 0, sum(a.new_user_total_amount_num) / sum(a.reg_num), 0), 4) today_amount_rate,
                    
                    round(if(sum(a.buy_first_new_user_amount_count) > 0, sum(a.buy_first_new_user_amount) / sum(a.buy_first_new_user_amount_count), 0), 2) buy_first_avg_amount,
                    round(if(sum(a.buy_new_user_total_amount_count) > 0, sum(a.buy_new_user_total_amount) / sum(a.buy_new_user_total_amount_count), 0), 2) buy_today_avg_amount,
                    
                    round(if(sum(a.nature_first_new_user_amount_count) > 0, sum(a.nature_first_new_user_amount) / sum(a.nature_first_new_user_amount_count), 0), 2) nature_first_avg_amount,
                    round(if(sum(a.nature_new_user_total_amount_count) > 0, sum(a.nature_new_user_total_amount) / sum(a.nature_new_user_total_amount_count), 0), 2) nature_today_avg_amount,
                    
                    round(if(sum(a.first_new_user_amount_count) > 0, sum(a.first_new_user_amount) / sum(a.first_new_user_amount_count), 0), 2) first_avg_amount,
                    round(if(sum(a.new_user_total_amount_count) > 0, sum(a.new_user_total_amount) / sum(a.new_user_total_amount_count), 0), 2) today_avg_amount,
                    
                    
                    round(if(sum(a.buy_new_user_total_amount_num) > 0 , sum(a.buy_reg_order_user_again) / sum(a.buy_new_user_total_amount_num), 0), 4) buy_today_again_rate,
                    round(if(sum(a.nature_new_user_total_amount_num) > 0 , sum(a.nature_reg_order_user_again) / sum(a.nature_new_user_total_amount_num), 0), 4) nature_today_again_rate,
                    round(if(sum(a.new_user_total_amount_num) > 0 , sum(a.reg_order_user_again) / sum(a.new_user_total_amount_num), 0), 4) today_again_rate,
                    
                    round(if(sum(a.buy_reg_num) > 0 , sum(a.buy_new_user_total_amount) / sum(a.buy_reg_num), 0), 2) buy_reg_user_arpu,
                    round(if(sum(a.buy_first_new_user_amount_num) > 0 , sum(a.buy_first_new_user_amount) / sum(a.buy_first_new_user_amount_num), 0), 2) buy_first_amount_arpu,
                    round(if(sum(a.buy_new_user_total_amount_num) > 0 , sum(a.buy_new_user_total_amount) / sum(a.buy_new_user_total_amount_num), 0), 2) buy_today_amount_arpu,
                    
                    round(if(sum(a.nature_reg_num) > 0 , sum(a.nature_new_user_total_amount) / sum(a.nature_reg_num), 0), 2) nature_reg_user_arpu,
                    round(if(sum(a.nature_first_new_user_amount_num) > 0 , sum(a.nature_first_new_user_amount) / sum(a.nature_first_new_user_amount_num), 0), 2) nature_first_amount_arpu,
                    round(if(sum(a.nature_new_user_total_amount_num) > 0 , sum(a.nature_new_user_total_amount) / sum(a.nature_new_user_total_amount_num), 0), 2) nature_today_amount_arpu,
                    
                    round(if(sum(a.reg_num) > 0 , sum(a.new_user_total_amount) / sum(a.reg_num), 0), 2) reg_user_arpu,
                    round(if(sum(a.first_new_user_amount_num) > 0 , sum(a.first_new_user_amount) / sum(a.first_new_user_amount_num), 0), 2) first_amount_arpu,
                    round(if(sum(a.new_user_total_amount_num) > 0 , sum(a.new_user_total_amount) / sum(a.new_user_total_amount_num), 0), 2) today_amount_arpu,
                    
                    round(if(SUM(buy_reg_num) > 0, SUM(cost) / SUM(buy_reg_num), 0), 2) buy_reg_cost,
                    round(if(SUM(nature_reg_num) > 0, SUM(cost) / SUM(nature_reg_num), 0), 2) nature_reg_cost,
                    round(if(SUM(reg_num) > 0, SUM(cost) / SUM(reg_num), 0), 2) reg_cost,
                    
                    round(if(SUM(cost) > 0 , SUM(buy_new_user_total_amount) / SUM(cost), 0), 4) buy_total_roi,
                    round(if(SUM(cost) > 0 , SUM(nature_new_user_total_amount) / SUM(cost), 0), 4) nature_total_roi,
                    round(if(SUM(cost) > 0 , SUM(new_user_total_amount) / SUM(cost), 0), 4) total_roi,
                    
                    round(if(SUM(buy_first_new_user_amount_num) > 0, SUM(cost) / SUM(buy_first_new_user_amount_num), 0), 2) buy_first_new_user_recharge_cost,
                    round(if(SUM(nature_first_new_user_amount_num) > 0, SUM(cost) / SUM(nature_first_new_user_amount_num), 0), 2) nature_first_new_user_recharge_cost,
                    round(if(SUM(first_new_user_amount_num) > 0, SUM(cost) / SUM(first_new_user_amount_num), 0), 2) first_new_user_recharge_cost,
                    
                    round(if(SUM(buy_new_user_total_amount_num) > 0, SUM(cost) / SUM(buy_new_user_total_amount_num), 0), 2) buy_total_recharge_cost,
                    round(if(SUM(nature_new_user_total_amount_num) > 0, SUM(cost) / SUM(nature_new_user_total_amount_num), 0), 2) nature_total_recharge_cost,
                    round(if(SUM(new_user_total_amount_num) > 0, SUM(cost) / SUM(new_user_total_amount_num), 0), 2) total_recharge_cost,
                    
                    SUM(buy_hundred_user_num) buy_hundred_user_num,
                    round(IF(SUM(buy_hundred_user_num) > 0, SUM(cost) / SUM(buy_hundred_user_num), 0), 2) buy_hundred_user_num_cost,
                    SUM(buy_first_role_num) buy_first_role_num,
                    SUM(buy_role_num) buy_role_num,
                    SUM(buy_new_user_total_role_num) buy_new_user_total_role_num,
                    round(IF(SUM(buy_first_role_num) > 0, SUM(cost) / SUM(buy_first_role_num), 0), 2) buy_first_role_num_cost,
                    round(IF(SUM(buy_role_num) > 0, SUM(cost) / SUM(buy_role_num), 0), 2) buy_role_num_cost,
                    round(IF(SUM(buy_new_user_total_role_num) >0, SUM(cost) / SUM(buy_new_user_total_role_num), 0), 2) buy_new_user_total_role_num_cost,
                    round(IF(SUM(buy_reg_num) >0, SUM(buy_first_role_num) / SUM(buy_reg_num), 0), 4) buy_first_role_num_rate,
                    round(IF(SUM(buy_reg_num) >0, SUM(buy_role_num) / SUM(buy_reg_num), 0), 4) buy_role_num_rate,
                    round(IF(SUM(buy_reg_num) >0, SUM(buy_new_user_total_role_num) / SUM(buy_reg_num), 0), 4) buy_new_user_total_role_num_rate,
                    
                    SUM(nature_hundred_user_num) nature_hundred_user_num,
                    round(IF(SUM(nature_hundred_user_num) > 0, SUM(cost) / SUM(nature_hundred_user_num), 0), 2) nature_hundred_user_num_cost,
                    SUM(nature_first_role_num) nature_first_role_num,
                    SUM(nature_role_num) nature_role_num,
                    SUM(nature_new_user_total_role_num) nature_new_user_total_role_num,
                    round(IF(SUM(nature_first_role_num) > 0, SUM(cost) / SUM(nature_first_role_num), 0), 2) nature_first_role_num_cost,
                    round(IF(SUM(nature_role_num) > 0, SUM(cost) / SUM(nature_role_num), 0), 2) nature_role_num_cost,
                    round(IF(SUM(nature_new_user_total_role_num) >0, SUM(cost) / SUM(nature_new_user_total_role_num), 0), 2) nature_new_user_total_role_num_cost,
                    round(IF(SUM(nature_reg_num) >0, SUM(nature_first_role_num) / SUM(nature_reg_num), 0), 4) nature_first_role_num_rate,
                    round(IF(SUM(nature_reg_num) >0, SUM(nature_role_num) / SUM(nature_reg_num), 0), 4) nature_role_num_rate,
                    round(IF(SUM(nature_reg_num) >0, SUM(nature_new_user_total_role_num) / SUM(nature_reg_num), 0), 4) nature_new_user_total_role_num_rate,
                   
                    SUM(hundred_user_num) hundred_user_num,
                    round(IF(SUM(hundred_user_num) > 0, SUM(cost) / SUM(hundred_user_num), 0), 2) hundred_user_num_cost,
                    SUM(first_role_num) first_role_num,
                    SUM(role_num) role_num,
                    SUM(new_user_total_role_num) new_user_total_role_num,
                    round(IF(SUM(first_role_num) > 0, SUM(cost) / SUM(first_role_num), 0), 2) first_role_num_cost,
                    round(IF(SUM(role_num) > 0, SUM(cost) / SUM(role_num), 0), 2) role_num_cost,
                    round(IF(SUM(new_user_total_role_num) >0, SUM(cost) / SUM(new_user_total_role_num), 0), 2) new_user_total_role_num_cost,
                    round(IF(SUM(reg_num) >0, SUM(first_role_num) / SUM(reg_num), 0), 4) first_role_num_rate,
                    round(IF(SUM(reg_num) >0, SUM(role_num) / SUM(reg_num), 0), 4) role_num_rate,
                    round(IF(SUM(reg_num) >0, SUM(new_user_total_role_num) / SUM(reg_num), 0), 4) new_user_total_role_num_rate
                    
                FROM
                    ads_game_day a
                """;
    }

    /**
     * 游戏总数据总计一栏sql
     *
     * @return String
     */
    private String gameDataTotalTotalSql() {
        return """
                SELECT
                    IFNULL(sum(buy_reg_num), 0) buy_reg_num,
                    IFNULL(sum(nature_reg_num), 0) nature_reg_num,
                    IFNULL(sum(reg_num), 0) reg_num,
                    IFNULL(sum(cost), 0) cost,
                    
                    IFNULL(sum(buy_first_new_user_amount_count), 0) buy_first_new_user_amount_count,
                    IFNULL(sum(buy_first_new_user_amount_num), 0) buy_first_new_user_amount_num,
                    IFNULL(sum(buy_first_new_user_amount), 0) buy_first_new_user_amount,
                    IFNULL(sum(nature_first_new_user_amount_count), 0) nature_first_new_user_amount_count,
                    IFNULL(sum(nature_first_new_user_amount_num), 0) nature_first_new_user_amount_num,
                    IFNULL(sum(nature_first_new_user_amount), 0) nature_first_new_user_amount,
                    IFNULL(sum(first_new_user_amount_count), 0) first_new_user_amount_count,
                    IFNULL(sum(first_new_user_amount_num), 0) first_new_user_amount_num,
                    IFNULL(sum(first_new_user_amount), 0) first_new_user_amount,
                    
                    IFNULL(sum(buy_new_user_total_amount_count), 0) buy_new_user_total_amount_count,
                    IFNULL(sum(buy_new_user_total_amount_num), 0) buy_new_user_total_amount_num,
                    IFNULL(sum(buy_new_user_total_amount), 0) buy_new_user_total_amount,
                    IFNULL(sum(nature_new_user_total_amount_count), 0) nature_new_user_total_amount_count,
                    IFNULL(sum(nature_new_user_total_amount_num), 0) nature_new_user_total_amount_num,
                    IFNULL(sum(nature_new_user_total_amount), 0) nature_new_user_total_amount,
                    IFNULL(sum(new_user_total_amount_count), 0) new_user_total_amount_count,
                    IFNULL(sum(new_user_total_amount_num), 0) new_user_total_amount_num,
                    IFNULL(sum(new_user_total_amount), 0) new_user_total_amount,
                    
                    round(if(SUM(cost) > 0 , SUM(buy_first_new_user_amount) / SUM(cost) ,0), 4) buy_first_roi,
                    round(if(sum(buy_reg_num) > 0 ,sum(buy_first_new_user_amount_num) / sum(buy_reg_num), 0), 4) buy_first_amount_rate,
                    round(if(sum(buy_reg_num) > 0, sum(buy_new_user_total_amount_num) / sum(buy_reg_num), 0), 4) buy_today_amount_rate,
                    round(if(SUM(cost) > 0 , SUM(nature_first_new_user_amount) / SUM(cost) ,0), 4) nature_first_roi,
                    round(if(sum(nature_reg_num) > 0 ,sum(nature_first_new_user_amount_num) / sum(nature_reg_num), 0), 4) nature_first_amount_rate,
                    round(if(sum(nature_reg_num) > 0, sum(nature_new_user_total_amount_num) / sum(nature_reg_num), 0), 4) nature_today_amount_rate,
                    round(if(SUM(cost) > 0 , SUM(first_new_user_amount) / SUM(cost) ,0), 4) first_roi,
                    round(if(sum(reg_num) > 0 ,sum(first_new_user_amount_num) / sum(reg_num), 0), 4) first_amount_rate,
                    round(if(sum(reg_num) > 0, sum(new_user_total_amount_num) / sum(reg_num), 0), 4) today_amount_rate,
                    
                    round(if(sum(buy_first_new_user_amount_count) > 0, sum(buy_first_new_user_amount) / sum(buy_first_new_user_amount_count), 0), 2) buy_first_avg_amount,
                    round(if(sum(buy_new_user_total_amount_count) > 0, sum(buy_new_user_total_amount) / sum(buy_new_user_total_amount_count), 0), 2) buy_today_avg_amount,
                    
                    round(if(sum(nature_first_new_user_amount_count) > 0, sum(nature_first_new_user_amount) / sum(nature_first_new_user_amount_count), 0), 2) nature_first_avg_amount,
                    round(if(sum(nature_new_user_total_amount_count) > 0, sum(nature_new_user_total_amount) / sum(nature_new_user_total_amount_count), 0), 2) nature_today_avg_amount,
                    
                    round(if(sum(first_new_user_amount_count) > 0, sum(first_new_user_amount) / sum(first_new_user_amount_count), 0), 2) first_avg_amount,
                    round(if(sum(new_user_total_amount_count) > 0, sum(new_user_total_amount) / sum(new_user_total_amount_count), 0), 2) today_avg_amount,
                    
                    
                    round(if(sum(buy_new_user_total_amount_num) > 0 , sum(buy_reg_order_user_again) / sum(buy_new_user_total_amount_num), 0), 4) buy_today_again_rate,
                    round(if(sum(nature_new_user_total_amount_num) > 0 , sum(nature_reg_order_user_again) / sum(nature_new_user_total_amount_num), 0), 4) nature_today_again_rate,
                    round(if(sum(new_user_total_amount_num) > 0 , sum(reg_order_user_again) / sum(new_user_total_amount_num), 0), 4) today_again_rate,
                    
                    round(if(sum(buy_reg_num) > 0 , sum(buy_new_user_total_amount) / sum(buy_reg_num), 0), 2) buy_reg_user_arpu,
                    round(if(sum(buy_first_new_user_amount_num) > 0 , sum(buy_first_new_user_amount) / sum(buy_first_new_user_amount_num), 0), 2) buy_first_amount_arpu,
                    round(if(sum(buy_new_user_total_amount_num) > 0 , sum(buy_new_user_total_amount) / sum(buy_new_user_total_amount_num), 0), 2) buy_today_amount_arpu,
                    
                    round(if(sum(nature_reg_num) > 0 , sum(nature_new_user_total_amount) / sum(nature_reg_num), 0), 2) nature_reg_user_arpu,
                    round(if(sum(nature_first_new_user_amount_num) > 0 , sum(nature_first_new_user_amount) / sum(nature_first_new_user_amount_num), 0), 2) nature_first_amount_arpu,
                    round(if(sum(nature_new_user_total_amount_num) > 0 , sum(nature_new_user_total_amount) / sum(nature_new_user_total_amount_num), 0), 2) nature_today_amount_arpu,
                    
                    round(if(sum(reg_num) > 0 , sum(new_user_total_amount) / sum(reg_num), 0), 2) reg_user_arpu,
                    round(if(sum(first_new_user_amount_num) > 0 , sum(first_new_user_amount) / sum(first_new_user_amount_num), 0), 2) first_amount_arpu,
                    round(if(sum(new_user_total_amount_num) > 0 , sum(new_user_total_amount) / sum(new_user_total_amount_num), 0), 2) today_amount_arpu,
                    
                    round(if(SUM(buy_reg_num) > 0, SUM(cost) / SUM(buy_reg_num), 0), 2) buy_reg_cost,
                    round(if(SUM(nature_reg_num) > 0, SUM(cost) / SUM(nature_reg_num), 0), 2) nature_reg_cost,
                    round(if(SUM(reg_num) > 0, SUM(cost) / SUM(reg_num), 0), 2) reg_cost,
                    
                    round(if(SUM(cost) > 0 , SUM(buy_new_user_total_amount) / SUM(cost), 0), 4) buy_total_roi,
                    round(if(SUM(cost) > 0 , SUM(nature_new_user_total_amount) / SUM(cost), 0), 4) nature_total_roi,
                    round(if(SUM(cost) > 0 , SUM(new_user_total_amount) / SUM(cost), 0), 4) total_roi,
                    
                    round(if(SUM(buy_first_new_user_amount_num) > 0, SUM(cost) / SUM(buy_first_new_user_amount_num), 0), 2) buy_first_new_user_recharge_cost,
                    round(if(SUM(nature_first_new_user_amount_num) > 0, SUM(cost) / SUM(nature_first_new_user_amount_num), 0), 2) nature_first_new_user_recharge_cost,
                    round(if(SUM(first_new_user_amount_num) > 0, SUM(cost) / SUM(first_new_user_amount_num), 0), 2) first_new_user_recharge_cost,
                    
                    round(if(SUM(buy_new_user_total_amount_num) > 0, SUM(cost) / SUM(buy_new_user_total_amount_num), 0), 2) buy_total_recharge_cost,
                    round(if(SUM(nature_new_user_total_amount_num) > 0, SUM(cost) / SUM(nature_new_user_total_amount_num), 0), 2) nature_total_recharge_cost,
                    round(if(SUM(new_user_total_amount_num) > 0, SUM(cost) / SUM(new_user_total_amount_num), 0), 2) total_recharge_cost,
                    
                    SUM(buy_hundred_user_num) buy_hundred_user_num,
                    round(IF(SUM(buy_hundred_user_num) > 0, SUM(cost) / SUM(buy_hundred_user_num), 0), 2) buy_hundred_user_num_cost,
                    SUM(buy_first_role_num) buy_first_role_num,
                    SUM(buy_role_num) buy_role_num,
                    SUM(buy_new_user_total_role_num) buy_new_user_total_role_num,
                    round(IF(SUM(buy_first_role_num) > 0, SUM(cost) / SUM(buy_first_role_num), 0), 2) buy_first_role_num_cost,
                    round(IF(SUM(buy_role_num) > 0, SUM(cost) / SUM(buy_role_num), 0), 2) buy_role_num_cost,
                    round(IF(SUM(buy_new_user_total_role_num) >0, SUM(cost) / SUM(buy_new_user_total_role_num), 0), 2) buy_new_user_total_role_num_cost,
                    round(IF(SUM(buy_reg_num) >0, SUM(buy_first_role_num) / SUM(buy_reg_num), 0), 4) buy_first_role_num_rate,
                    round(IF(SUM(buy_reg_num) >0, SUM(buy_role_num) / SUM(buy_reg_num), 0), 4) buy_role_num_rate,
                    round(IF(SUM(buy_reg_num) >0, SUM(buy_new_user_total_role_num) / SUM(buy_reg_num), 0), 4) buy_new_user_total_role_num_rate,
                    
                    SUM(nature_hundred_user_num) nature_hundred_user_num,
                    round(IF(SUM(nature_hundred_user_num) > 0, SUM(cost) / SUM(nature_hundred_user_num), 0), 2) nature_hundred_user_num_cost,
                    SUM(nature_first_role_num) nature_first_role_num,
                    SUM(nature_role_num) nature_role_num,
                    SUM(nature_new_user_total_role_num) nature_new_user_total_role_num,
                    round(IF(SUM(nature_first_role_num) > 0, SUM(cost) / SUM(nature_first_role_num), 0), 2) nature_first_role_num_cost,
                    round(IF(SUM(nature_role_num) > 0, SUM(cost) / SUM(nature_role_num), 0), 2) nature_role_num_cost,
                    round(IF(SUM(nature_new_user_total_role_num) >0, SUM(cost) / SUM(nature_new_user_total_role_num), 0), 2) nature_new_user_total_role_num_cost,
                    round(IF(SUM(nature_reg_num) >0, SUM(nature_first_role_num) / SUM(nature_reg_num), 0), 4) nature_first_role_num_rate,
                    round(IF(SUM(nature_reg_num) >0, SUM(nature_role_num) / SUM(nature_reg_num), 0), 4) nature_role_num_rate,
                    round(IF(SUM(nature_reg_num) >0, SUM(nature_new_user_total_role_num) / SUM(nature_reg_num), 0), 4) nature_new_user_total_role_num_rate,
                   
                    SUM(hundred_user_num) hundred_user_num,
                    round(IF(SUM(hundred_user_num) > 0, SUM(cost) / SUM(hundred_user_num), 0), 2) hundred_user_num_cost,
                    SUM(first_role_num) first_role_num,
                    SUM(role_num) role_num,
                    SUM(new_user_total_role_num) new_user_total_role_num,
                    round(IF(SUM(first_role_num) > 0, SUM(cost) / SUM(first_role_num), 0), 2) first_role_num_cost,
                    round(IF(SUM(role_num) > 0, SUM(cost) / SUM(role_num), 0), 2) role_num_cost,
                    round(IF(SUM(new_user_total_role_num) >0, SUM(cost) / SUM(new_user_total_role_num), 0), 2) new_user_total_role_num_cost,
                    round(IF(SUM(reg_num) >0, SUM(first_role_num) / SUM(reg_num), 0), 4) first_role_num_rate,
                    round(IF(SUM(reg_num) >0, SUM(role_num) / SUM(reg_num), 0), 4) role_num_rate,
                    round(IF(SUM(reg_num) >0, SUM(new_user_total_role_num) / SUM(reg_num), 0), 4) new_user_total_role_num_rate
                    
                FROM
                    game_ads.ads_game_day
                """;
    }

    /**
     * 和充值时间有关的sql
     *
     * @return
     */
    private String gameDataTotalSqlRecharge() {
        return """
                SELECT
                    IFNULL(sum(buy_amount_count),0) buy_amount_count,
                    IFNULL(sum(buy_amount),0) buy_amount,
                    IFNULL(sum(nature_amount_count),0) nature_amount_count,
                    IFNULL(sum(nature_amount),0) nature_amount,
                    IFNULL(sum(amount_count),0) amount_count,
                    IFNULL(sum(amount),0) amount,
                    
                    round(if(sum(buy_amount_count) > 0, sum(buy_amount) / sum(buy_amount_count), 0), 2) buy_avg_amount,
                    round(if(sum(nature_amount_count) > 0, sum(nature_amount) / sum(nature_amount_count), 0), 2) nature_avg_amount,
                    round(if(sum(amount_count) > 0, sum(amount) / sum(amount_count), 0), 2) avg_amount
                FROM
                    game_ads.ads_game_day
                                
                """;
    }

    /**
     * 复充表sql（总量）
     *
     * @return String
     */
    private String adsGameDataAgainSql() {
        return """  
                SELECT
                    dt,
                    ddf,
                    game_id,
                    game_name,
                    c1,
                    c2,
                    c3,
                    c4,
                    c5,
                    c6,
                    c7,
                    c8,
                    c9
                FROM
                    ads_game_day_again
                """;
    }

    /**
     * 复充表sql（买量）
     *
     * @return String
     */
    private String adsGameDataAgainBuySql() {
        return """  
                SELECT
                    dt,
                    ddf,
                    game_id,
                    game_name,
                    buy_c1,
                    buy_c2,
                    buy_c3,
                    buy_c4,
                    buy_c5,
                    buy_c6,
                    buy_c7,
                    buy_c8,
                    buy_c9
                FROM
                    ads_game_day_again_buy
                """;
    }

    /**
     * 复充表sql（自然量）
     *
     * @return String
     */
    private String adsGameDataAgainNatureSql() {
        return """  
                SELECT
                    dt,
                    ddf,
                    game_id,
                    game_name,
                    nature_c1,
                    nature_c2,
                    nature_c3,
                    nature_c4,
                    nature_c5,
                    nature_c6,
                    nature_c7,
                    nature_c8,
                    nature_c9
                FROM
                    ads_game_day_again_nature
                """;
    }

    @Override
    public Page<GameDataH5VO> getH5Recharge(GameDataH5DTO dto) {
        if (null == dto.getCostBeginDate() || null == dto.getCostEndDate()) {
            dto.setCostEndDate(LocalDate.now());
            dto.setCostBeginDate(LocalDate.now());
        }
        if (null == dto.getRechargeBeginDate() || null == dto.getRechargeEndDate()) {
            dto.setRechargeEndDate(LocalDate.now());
            dto.setRechargeBeginDate(LocalDate.now());
        }
        if (StringUtils.isBlank(dto.getSortFiled())) {
            dto.setSortFiled("h5_nature_new_user_total_amount");
        }
        if (StringUtils.isBlank(dto.getSortType())) {
            dto.setSortType(OrderByEnum.DESC.getOrderType());
        }
        Pager pager = dao.createPager(dto.getPageNum(), dto.getPageSize());
        // 创建一个 Criteria 接口实例
        SimpleCriteria cri = Cnd.cri();
        if (StringUtils.isNotBlank(dto.getGameName())) {
            cri.where().andLike("a.game_name", dto.getGameName());
        }
        if (StringUtils.isNotBlank(dto.getSourceSystem())) {
            cri.where().andEquals("a.source_system", dto.getSourceSystem());
        }
        if (null != dto.getGameId()) {
            cri.where().andEquals("a.game_id", dto.getGameId());
        }
        cri.where().and("a.dt", ">=", dto.getCostBeginDate()).and("a.dt", "<=", dto.getCostEndDate());
        cri.where().andEquals("a.id", 1);
        cri.groupBy("a.source_system,a.game_id");
        cri.orderBy(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, dto.getSortFiled()), dto.getSortType());
        Sql sql = Sqls.queryEntity(h5RechargeSql() + "$condition");
        sql.setPager(pager);
        Entity<GameDataH5VO> entity = dao.getEntity(GameDataH5VO.class);
        sql.setEntity(entity).setCondition(cri);
        sql.setParam("rechargeBeginDay", dto.getRechargeBeginDate());
        sql.setParam("rechargeEndDay", dto.getRechargeEndDate());
        dao.execute(sql);
        Sql sqlCount = Sqls.queryEntity("select count(*) from ads_h5_game_day a " + "$condition");
        sqlCount.setCondition(cri);
        pager.setRecordCount((int) Daos.queryCount(dao, sqlCount));
        List<Long> gameIds = sql.getList(GameDataH5VO.class).stream()
                .map(GameDataH5VO::getGameId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(gameIds)) {
            return new Page<>();
        }

        //h5注册时间内的dayn充值数据，dayn的json数据
        Map<String, Object> dayNMap = new HashMap<>(4);
        dayNMap.put("beginDay", dto.getCostBeginDate());
        dayNMap.put("endDay", dto.getCostEndDate());

        //h5导量注册时间内的dayn充值数据，dayn的json数据
        Map<String, Object> importDayNMap = new HashMap<>(4);
        importDayNMap.put("beginDay", dto.getCostBeginDate());
        importDayNMap.put("endDay", dto.getCostEndDate());

        //h5账面
        String show = """
                select
                sum(a.h5_buy_amount) as h5_buy_amount,
                (select count(distinct user_id) from ads_information
                where
                source_system = a.source_system
                and h5_game_id = a.game_id
                and agent_id != 0
                and classify = 5
                and order_time >= @rechargeBeginDay
                and order_time <= @rechargeEndDay) as h5_buy_amount_num,
                sum(a.h5_nature_amount) as h5_nature_amount,
                (select count(distinct user_id) from ads_information
                where
                source_system = a.source_system
                and h5_game_id = a.game_id
                and agent_id = 0
                and classify = 5
                and order_time >= @rechargeBeginDay
                and order_time <= @rechargeEndDay) as h5_nature_amount_num
                from ads_h5_game_day a
                """;
        Sql showSql = Sqls.queryEntity(show + "$condition");
        showSql.setParam("rechargeBeginDay", dto.getRechargeBeginDate());
        showSql.setParam("rechargeEndDay", dto.getRechargeEndDate());
        Entity<ShowRecharge> showEntity = dao.getEntity(ShowRecharge.class);
        List<GameDataH5VO> gameDataH5VOList = sql.getList(GameDataH5VO.class).stream().map(item -> {
            {
                SimpleCriteria showCri = Cnd.cri();
                if (StringUtils.isNotBlank(dto.getGameName())) {
                    showCri.where().andLike("a.game_name", dto.getGameName());
                }
                showCri.where().and("a.dt", ">=", dto.getRechargeBeginDate()).and("a.dt", "<=", dto.getRechargeEndDate());
                showCri.groupBy("a.source_system,a.game_id");
                Long h5GameId = item.getGameId();
                showCri.where().andEquals("a.game_id", h5GameId);
                showCri.where().andEquals("a.id", 1);
                String sourceSystem = item.getSourceSystem();
                showCri.where().andEquals("a.source_system", sourceSystem);
                showSql.setEntity(showEntity).setCondition(showCri);
                dao.execute(showSql);
                ShowRecharge showRecharge = showSql.getObject(ShowRecharge.class);
                if (null != showRecharge) {
                    item.setH5NatureAmount(showRecharge.getH5NatureAmount());
                    item.setH5NatureAmountNum(showRecharge.getH5NatureAmountNum());
                    item.setH5BuyAmount(showRecharge.getH5BuyAmount());
                    item.setH5BuyAmountNum(showRecharge.getH5BuyAmountNum());
                } else {
                    item.setH5NatureAmount(BigDecimal.ZERO);
                    item.setH5NatureAmountNum(0L);
                    item.setH5BuyAmount(BigDecimal.ZERO);
                    item.setH5BuyAmountNum(0L);
                }

                dayNMap.put("sourceSystem", sourceSystem);
                dayNMap.put("h5GameId", h5GameId);

                H5DayN h5PayDayN = getH5PayDayN(dto, dayNMap);
                //h5充值时间范围内存在买量充值数据
                if (null != h5PayDayN) {
                    item.setH5NewUserRechargeMoney(h5PayDayN.getRechargeMoney());
                    item.setH5NewUserRechargeNum(h5PayDayN.getRechargeNum());
                } else {
                    item.setH5NewUserRechargeMoney(BigDecimal.ZERO);
                    item.setH5NewUserRechargeNum(0L);
                }
                H5DayN h5NatureDayN = getNatureH5DayN(dto, dayNMap);
                //h5充值时间范围内存在自然量充值数据
                if (null != h5NatureDayN) {
                    item.setH5NewUserNatureRechargeMoney(h5NatureDayN.getRechargeMoney());
                    item.setH5NewUserNatureRechargeNum(h5NatureDayN.getRechargeNum());
                } else {
                    item.setH5NewUserNatureRechargeMoney(BigDecimal.ZERO);
                    item.setH5NewUserNatureRechargeNum(0L);
                }
                item.setH5NewUserNatureRechargeRate(null == item.getH5NewUserNatureRechargeMoney() || null == item.getH5NewUserRechargeMoney() || item.getH5NewUserRechargeMoney().compareTo(BigDecimal.ZERO) == 0 ?
                        BigDecimal.ZERO : item.getH5NewUserNatureRechargeMoney().divide(item.getH5NewUserRechargeMoney(), 4, RoundingMode.DOWN));
                item.setH5NewUserNatureNewUserRate(null == item.getH5NewUserRechargeNum() || null == item.getH5NewUserNatureRechargeNum() || item.getH5NewUserRechargeNum() == 0 ?
                        BigDecimal.ZERO : BigDecimal.valueOf(item.getH5NewUserNatureRechargeNum()).divide(BigDecimal.valueOf(item.getH5NewUserRechargeNum()), 4, RoundingMode.DOWN));

                //买量导入用户数据
                List<H5BuyUserVO> buyGameList = getH5BuyUserVOList(dto, importDayNMap, item);
                //官方导入用户数据
                List<H5NatureUserVO> natureGameList = getH5NatureUserVOList(dto, importDayNMap, item);

                item.setH5BuyUserVOList(buyGameList);
                item.setH5NatureUserVOList(natureGameList);
                return item;
            }
        }).collect(Collectors.toList());

        return new Page<>(gameDataH5VOList, pager);
    }

    private List<H5NatureUserVO> getH5NatureUserVOList(GameDataH5DTO dto, Map<String, Object> importDayNMap, GameDataH5VO item) {
        Sql natureGameSql = Sqls.queryEntity("""
                select
                nature_source_system,
                nature_game_id,
                max(nature_game_name) as nature_game_name,
                max(nature_classify) as nature_classify,
                sum(h5_lead_nature_new_user_total_amount) as h5_lead_nature_new_user_total_amount,
                sum(h5_lead_nature_new_user_total_amount_num) as h5_lead_nature_new_user_total_amount_num
                from ads_h5_game_day
                where
                game_id = @gameId
                and
                dt>=@beginDay and dt<=@endDay
                group by nature_source_system,nature_game_id
                """);
        natureGameSql.setParam("gameId", item.getGameId());
        natureGameSql.setParam("beginDay", dto.getCostBeginDate());
        natureGameSql.setParam("endDay", dto.getCostEndDate());
        Entity<H5NatureUserVO> natureGameEntity = dao.getEntity(H5NatureUserVO.class);
        natureGameSql.setEntity(natureGameEntity);
        dao.execute(natureGameSql);

        Sql natureShowGameSql = Sqls.queryEntity("""
                select
                a.nature_source_system,
                a.nature_game_id,
                sum(a.h5_lead_nature_amount) as h5_lead_nature_amount,
                (select count(distinct user_id) from ads_information
                where
                source_system = a.nature_source_system
                and game_id = a.nature_game_id
                and order_h5_game_id = @gameId
                and agent_id = 0
                and classify != 5
                and order_game_classify = 5
                and order_time >= @beginDay
                and order_time <= @endDay) as h5_lead_nature_amount_num
                from ads_h5_game_day a
                where
                a.game_id = @gameId
                and
                a.dt>=@beginDay and a.dt<=@endDay
                group by a.nature_source_system,a.nature_game_id
                """);
        natureShowGameSql.setParam("gameId", item.getGameId());
        natureShowGameSql.setParam("beginDay", dto.getRechargeBeginDate());
        natureShowGameSql.setParam("endDay", dto.getRechargeEndDate());
        Entity<H5NatureUserVO> buyShowGameEntity = dao.getEntity(H5NatureUserVO.class);
        natureShowGameSql.setEntity(buyShowGameEntity);
        dao.execute(natureShowGameSql);
        Map<Long, List<H5NatureUserVO>> showBuyGameMap = natureShowGameSql.getList(H5NatureUserVO.class).stream()
                .filter(tt -> null != tt.getNatureGameId())
                .collect(Collectors.groupingBy(H5NatureUserVO::getNatureGameId));

        Map<String,Object> natureNewUserMap = new HashMap<>(7);
        natureNewUserMap.put("h5GameId",item.getGameId());
        natureNewUserMap.put("rechargeBeginDay",dto.getRechargeBeginDate());
        natureNewUserMap.put("rechargeEndDay",dto.getRechargeEndDate());
        natureNewUserMap.put("costBeginDay",dto.getCostBeginDate());
        natureNewUserMap.put("costEndDay",dto.getCostEndDate());
        List<H5NatureUserVO> natureGameList = natureGameSql.getList(H5NatureUserVO.class).stream()
                .filter(tt -> null != tt.getNatureGameId())
                .map(natureGame -> {
                    natureNewUserMap.put("natureSourceSystem",natureGame.getNatureSourceSystem());
                    natureNewUserMap.put("natureGameId",natureGame.getNatureGameId());

                    importDayNMap.put("sourceSystem", natureGame.getNatureSourceSystem());
                    importDayNMap.put("gameId", natureGame.getNatureGameId());
                    H5DayN importNatureH5DayN = getImportNatureH5DayN(dto, importDayNMap, natureNewUserMap);
                    if (null != showBuyGameMap && null != showBuyGameMap.get(natureGame.getNatureGameId()) && !showBuyGameMap.get(natureGame.getNatureGameId()).isEmpty()) {
                        natureGame.setH5LeadNatureAmount(showBuyGameMap.get(natureGame.getNatureGameId()).get(0).getH5LeadNatureAmount());
                        natureGame.setH5LeadNatureAmountNum(showBuyGameMap.get(natureGame.getNatureGameId()).get(0).getH5LeadNatureAmountNum());
                    } else {
                        natureGame.setH5LeadNatureAmount(BigDecimal.ZERO);
                        natureGame.setH5LeadNatureAmountNum(0L);
                    }
                    //充值时间范围内存在官方导量充值数据
                    if (null != importNatureH5DayN) {
                        natureGame.setNewUserRechargeMoney(importNatureH5DayN.getRechargeMoney());
                        natureGame.setNewUserRechargeNum(importNatureH5DayN.getRechargeNum());
                    } else {
                        natureGame.setNewUserRechargeMoney(BigDecimal.ZERO);
                        natureGame.setNewUserRechargeNum(0L);
                    }
                    return natureGame;
                }).collect(Collectors.toList());
        return natureGameList;
    }

    private List<H5BuyUserVO> getH5BuyUserVOList(GameDataH5DTO dto, Map<String, Object> importDayNMap, GameDataH5VO item) {
        //买量导入游戏
        Sql buyGameSql = Sqls.queryEntity("""
                select
                buy_source_system,
                buy_game_id,
                max(buy_game_name) as buy_game_name,
                max(buy_classify) as buy_classify,
                sum(h5_lead_buy_new_user_total_amount) as h5_lead_buy_new_user_total_amount,
                sum(h5_lead_buy_new_user_total_amount_num) as h5_lead_buy_new_user_total_amount_num
                from ads_h5_game_day
                where
                game_id = @gameId
                and
                dt>=@beginDay and dt<=@endDay
                group by buy_source_system,buy_game_id
                """);
        buyGameSql.setParam("gameId", item.getGameId());
        buyGameSql.setParam("beginDay", dto.getCostBeginDate());
        buyGameSql.setParam("endDay", dto.getCostEndDate());
        Entity<H5BuyUserVO> buyGameEntity = dao.getEntity(H5BuyUserVO.class);
        buyGameSql.setEntity(buyGameEntity);
        dao.execute(buyGameSql);

        //买量导入游戏账面
        Sql buyShowGameSql = Sqls.queryEntity("""
                select
                a.buy_source_system as buy_source_system,
                a.buy_game_id as buy_game_id,
                sum(a.h5_lead_buy_amount) as h5_lead_buy_amount,
                (select count(distinct user_id) from ads_information
                where
                source_system = a.buy_source_system
                and game_id = a.buy_game_id
                and order_h5_game_id = @gameId
                and agent_id != 0
                and classify != 5
                and order_game_classify = 5
                and order_time >= @beginDay
                and order_time <= @endDay) as h5_lead_buy_amount_num
                from ads_h5_game_day a
                where
                a.game_id = @gameId
                and
                a.dt>=@beginDay and a.dt<=@endDay
                group by a.buy_source_system,a.buy_game_id
                """);
        buyShowGameSql.setParam("gameId", item.getGameId());
        buyShowGameSql.setParam("beginDay", dto.getRechargeBeginDate());
        buyShowGameSql.setParam("endDay", dto.getRechargeEndDate());
        Entity<H5BuyUserVO> buyShowGameEntity = dao.getEntity(H5BuyUserVO.class);
        buyShowGameSql.setEntity(buyShowGameEntity);
        dao.execute(buyShowGameSql);
        Map<Long, List<H5BuyUserVO>> showBuyGameMap = buyShowGameSql.getList(H5BuyUserVO.class).stream()
                .filter(tt -> null != tt.getBuyGameId())
                .collect(Collectors.groupingBy(H5BuyUserVO::getBuyGameId));

        Map<String,Object> buyNewUserMap = new HashMap<>(7);
        buyNewUserMap.put("h5GameId",item.getGameId());
        buyNewUserMap.put("rechargeBeginDay",dto.getRechargeBeginDate());
        buyNewUserMap.put("rechargeEndDay",dto.getRechargeEndDate());
        buyNewUserMap.put("costBeginDay",dto.getCostBeginDate());
        buyNewUserMap.put("costEndDay",dto.getCostEndDate());
        List<H5BuyUserVO> buyGameList = buyGameSql.getList(H5BuyUserVO.class).stream()
                .filter(tt -> null != tt.getBuyGameId())
                .map(buyGame -> {
                    importDayNMap.put("sourceSystem", buyGame.getBuySourceSystem());
                    importDayNMap.put("gameId", buyGame.getBuyGameId());

                    buyNewUserMap.put("buySourceSystem",buyGame.getBuySourceSystem());
                    buyNewUserMap.put("buyGameId",buyGame.getBuyGameId());
                    H5DayN importBuyH5DayN = getImportBuyH5DayN(dto, importDayNMap, buyNewUserMap);
                    if (null != showBuyGameMap && null != showBuyGameMap.get(buyGame.getBuyGameId()) && !showBuyGameMap.get(buyGame.getBuyGameId()).isEmpty()) {
                        buyGame.setH5LeadBuyAmount(showBuyGameMap.get(buyGame.getBuyGameId()).get(0).getH5LeadBuyAmount());
                        buyGame.setH5LeadBuyAmountNum(showBuyGameMap.get(buyGame.getBuyGameId()).get(0).getH5LeadBuyAmountNum());
                    } else {
                        buyGame.setH5LeadBuyAmount(BigDecimal.ZERO);
                        buyGame.setH5LeadBuyAmountNum(0L);
                    }
                    //买量导入游戏充值时间范围内存在新用户充值数据
                    if (null != importBuyH5DayN) {
                        buyGame.setNewUserRechargeMoney(importBuyH5DayN.getRechargeMoney());
                        buyGame.setNewUserRechargeNum(importBuyH5DayN.getRechargeNum());
                    } else {
                        buyGame.setNewUserRechargeMoney(BigDecimal.ZERO);
                        buyGame.setNewUserRechargeNum(0L);
                    }
                    return buyGame;
                }).collect(Collectors.toList());
        return buyGameList;
    }

    private H5DayN getImportNatureH5DayN(GameDataH5DTO dto, Map<String, Object> dayNMap, Map<String, Object> natureNewUserMap) {
        Sql importNatureSqlDayN = Sqls.queryString("""
                select dayN from ads_dayn_h5_lead_nature_game
                where
                dt>=@beginDay and dt<=@endDay
                and source_system = @sourceSystem
                and h5_lead_nature_game_id = @gameId
                """);
        importNatureSqlDayN.setParams(dayNMap);
        dao.execute(importNatureSqlDayN);
        List<String> importNatureDayNList = importNatureSqlDayN.getList(String.class);

        Sql newUserNum = Sqls.fetchLong("""
                select
                	count(DISTINCT user_id)
                from
                	ads_information
                where
                source_system = @natureSourceSystem
                and game_id = @natureGameId
                and order_h5_game_id = @h5GameId
                and agent_id = 0
                and classify != 5
                and order_game_classify = 5
                and order_time >= @rechargeBeginDay
                and order_time <= @rechargeEndDay
                and reg_time >= @costBeginDay
                and reg_time <= @costEndDay
                """);
        newUserNum.setParams(natureNewUserMap);
        dao.execute(newUserNum);
        Long newUserNumLong = newUserNum.getLong();

        H5DayN importNatureDayN = null;
        //ads_dayn_h5_nature_game 存在充值数据
        if (!CollectionUtils.isEmpty(importNatureDayNList)) {
            Gson gson = new Gson();
            List<Map<String, String>> jsonList = gson.fromJson(importNatureDayNList.toString(), ArrayList.class);

            Map<String, String> resultMap = new HashMap<>();
            resultMap.put("dayN", "0.00-0");
            for (Map<String, String> jsonMap : jsonList) {
                for (Map.Entry<String, String> entry : jsonMap.entrySet()) {
                    String key = entry.getKey();
                    String value = entry.getValue();
                    String[] parts = value.split("-");
                    if (DateUtil.parseLocalDate(key).compareTo(dto.getRechargeBeginDate()) >= 0
                            && DateUtil.parseLocalDate(key).compareTo(dto.getRechargeEndDate()) <= 0) {
                        String[] existingParts = resultMap.get("dayN").split("-");
                        BigDecimal money = new BigDecimal(existingParts[0]).add(new BigDecimal(parts[0]));
                        int num = Integer.parseInt(existingParts[1]) + Integer.parseInt(parts[1]);
                        String newValue = money + "-" + num;
                        resultMap.put("dayN", newValue);
                    }
                }
            }
            if (!resultMap.isEmpty() && !resultMap.get("dayN").isEmpty()) {
                String[] parts = resultMap.get("dayN").split("-");
                importNatureDayN = H5DayN.builder()
                        .rechargeMoney(new BigDecimal(parts[0]))
                        .rechargeNum(newUserNumLong)
                        .build();
            }
        }
        return importNatureDayN;
    }

    private H5DayN getImportBuyH5DayN(GameDataH5DTO dto, Map<String, Object> dayNMap, Map<String, Object> buyNewUserMap) {
        Sql importBuySqlDayN = Sqls.queryString("""
                select dayN from ads_dayn_h5_lead_buy_game
                where
                dt>=@beginDay and dt<=@endDay
                and source_system = @sourceSystem
                and h5_lead_buy_game_id = @gameId
                """);
        importBuySqlDayN.setParams(dayNMap);
        dao.execute(importBuySqlDayN);
        List<String> importBuyDayNList = importBuySqlDayN.getList(String.class);

        Sql newUserNum = Sqls.fetchLong("""
                select
                	count(DISTINCT user_id)
                from
                	ads_information
                where
                source_system = @buySourceSystem
                and game_id = @buyGameId
                and order_h5_game_id = @h5GameId
                and agent_id != 0
                and classify != 5
                and order_game_classify = 5
                and order_time >= @rechargeBeginDay
                and order_time <= @rechargeEndDay
                and reg_time >= @costBeginDay
                and reg_time <= @costEndDay
                """);
        newUserNum.setParams(buyNewUserMap);
        dao.execute(newUserNum);
        Long newUserNumLong = newUserNum.getLong();

        H5DayN importBuyDayN = null;
        //ads_dayn_h5_nature_game 存在充值数据
        if (!CollectionUtils.isEmpty(importBuyDayNList)) {
            Gson gson = new Gson();
            List<Map<String, String>> jsonList = gson.fromJson(importBuyDayNList.toString(), ArrayList.class);

            Map<String, String> resultMap = new HashMap<>();
            resultMap.put("dayN", "0.00-0");
            for (Map<String, String> jsonMap : jsonList) {
                for (Map.Entry<String, String> entry : jsonMap.entrySet()) {
                    String key = entry.getKey();
                    String value = entry.getValue();
                    String[] parts = value.split("-");
                    if (DateUtil.parseLocalDate(key).compareTo(dto.getRechargeBeginDate()) >= 0
                            && DateUtil.parseLocalDate(key).compareTo(dto.getRechargeEndDate()) <= 0) {
                        String[] existingParts = resultMap.get("dayN").split("-");
                        BigDecimal money = new BigDecimal(existingParts[0]).add(new BigDecimal(parts[0]));
                        int num = Integer.parseInt(existingParts[1]) + Integer.parseInt(parts[1]);
                        String newValue = money + "-" + num;
                        resultMap.put("dayN", newValue);
                    }
                }
            }
            if (!resultMap.isEmpty() && !resultMap.get("dayN").isEmpty()) {
                String[] parts = resultMap.get("dayN").split("-");
                importBuyDayN = H5DayN.builder()
                        .rechargeMoney(new BigDecimal(parts[0]))
                        .rechargeNum(newUserNumLong)
                        .build();
            }
        }
        return importBuyDayN;
    }

    private H5DayN getNatureH5DayN(GameDataH5DTO dto, Map<String, Object> dayNMap) {
        Sql natureSqlDayN = Sqls.queryString("""
                select dayN from ads_dayn_h5_nature_game
                where
                dt>=@beginDay and dt<=@endDay
                and source_system = @sourceSystem
                and h5_nature_game_id = @h5GameId
                """);
        natureSqlDayN.setParams(dayNMap);
        dao.execute(natureSqlDayN);
        List<String> natureDayNList = natureSqlDayN.getList(String.class);

        Sql newUserNum = Sqls.fetchLong("""
                select
                	count(DISTINCT user_id)
                from
                	ads_information
                where
                    source_system = @sourceSystem
                    and h5_game_id = @h5GameId
                    and agent_id = 0
                    and classify = 5
                    and order_time >= @rechargeBeginDay
                    and order_time <= @rechargeEndDay
                    and reg_time >= @costBeginDay
                    and reg_time <= @costEndDay
                """);
        Map<String, Object> newUserNumMap = new HashMap<>(6);
        newUserNumMap.put("costBeginDay", dto.getCostBeginDate());
        newUserNumMap.put("costEndDay", dto.getCostEndDate());
        newUserNumMap.put("rechargeBeginDay", dto.getRechargeBeginDate());
        newUserNumMap.put("rechargeEndDay", dto.getRechargeEndDate());

        H5DayN h5NatureDayN = null;
        //ads_dayn_h5_nature_game 存在充值数据
        if (!CollectionUtils.isEmpty(natureDayNList)) {
            Gson gson = new Gson();
            List<Map<String, String>> jsonList = gson.fromJson(natureDayNList.toString(), ArrayList.class);

            Map<String, String> resultMap = new HashMap<>();
            resultMap.put("dayN", "0.00-0");
            for (Map<String, String> jsonMap : jsonList) {
                for (Map.Entry<String, String> entry : jsonMap.entrySet()) {
                    String key = entry.getKey();
                    String value = entry.getValue();
                    String[] parts = value.split("-");
                    if (DateUtil.parseLocalDate(key).compareTo(dto.getRechargeBeginDate()) >= 0
                            && DateUtil.parseLocalDate(key).compareTo(dto.getRechargeEndDate()) <= 0) {
                        String[] existingParts = resultMap.get("dayN").split("-");
                        BigDecimal money = new BigDecimal(existingParts[0]).add(new BigDecimal(parts[0]));
                        int num = Integer.parseInt(existingParts[1]) + Integer.parseInt(parts[1]);
                        String newValue = money + "-" + num;
                        resultMap.put("dayN", newValue);
                    }
                }
            }
            newUserNumMap.put("sourceSystem", dayNMap.get("sourceSystem"));
            newUserNumMap.put("h5GameId", dayNMap.get("h5GameId"));
            newUserNum.setParams(newUserNumMap);
            dao.execute(newUserNum);
            Long newUserNumLong = newUserNum.getLong();
            if (!resultMap.isEmpty() && !resultMap.get("dayN").isEmpty()) {
                String[] parts = resultMap.get("dayN").split("-");
                h5NatureDayN = H5DayN.builder()
                        .rechargeMoney(new BigDecimal(parts[0]))
                        .rechargeNum(newUserNumLong)
                        .build();
            }
        }
        return h5NatureDayN;
    }

    private H5DayN getH5PayDayN(GameDataH5DTO dto, Map<String, Object> dayNMap) {
        H5DayN h5PayDayN = null;
        Sql paySqlDayN = Sqls.queryString("""
                select dayN from ads_dayn_h5_pay_game
                where
                dt>=@beginDay and dt<=@endDay
                and source_system = @sourceSystem
                and h5_game_id = @h5GameId
                """);
        paySqlDayN.setParams(dayNMap);
        dao.execute(paySqlDayN);
        List<String> payDayNList = paySqlDayN.getList(String.class);

        Sql newUserNum = Sqls.fetchLong("""
                select
                	count(DISTINCT user_id)
                from
                	ads_information
                where
                    source_system = @sourceSystem
                    and h5_game_id = @h5GameId
                    and agent_id != 0
                    and classify = 5
                    and order_time >= @rechargeBeginDay
                    and order_time <= @rechargeEndDay
                    and reg_time >= @costBeginDay
                    and reg_time <= @costEndDay
                """);
        Map<String, Object> newUserNumMap = new HashMap<>(6);
        newUserNumMap.put("costBeginDay", dto.getCostBeginDate());
        newUserNumMap.put("costEndDay", dto.getCostEndDate());
        newUserNumMap.put("rechargeBeginDay", dto.getRechargeBeginDate());
        newUserNumMap.put("rechargeEndDay", dto.getRechargeEndDate());
        //ads_dayn_h5_pay_game存在充值数据
        if (!CollectionUtils.isEmpty(payDayNList)) {
            Gson gson = new Gson();
            List<Map<String, String>> jsonList = gson.fromJson(payDayNList.toString(), ArrayList.class);

            Map<String, String> resultMap = new HashMap<>();
            resultMap.put("dayN", "0.00-0");
            for (Map<String, String> jsonMap : jsonList) {
                for (Map.Entry<String, String> entry : jsonMap.entrySet()) {
                    String key = entry.getKey();
                    String value = entry.getValue();
                    String[] parts = value.split("-");
                    if (DateUtil.parseLocalDate(key).compareTo(dto.getRechargeBeginDate()) >= 0
                            && DateUtil.parseLocalDate(key).compareTo(dto.getRechargeEndDate()) <= 0) {
                        String[] existingParts = resultMap.get("dayN").split("-");
                        BigDecimal money = new BigDecimal(existingParts[0]).add(new BigDecimal(parts[0]));
                        int num = Integer.parseInt(existingParts[1]) + Integer.parseInt(parts[1]);
                        String newValue = money + "-" + num;
                        resultMap.put("dayN", newValue);
                    }
                }
            }
            newUserNumMap.put("sourceSystem", dayNMap.get("sourceSystem"));
            newUserNumMap.put("h5GameId", dayNMap.get("h5GameId"));
            newUserNum.setParams(newUserNumMap);
            dao.execute(newUserNum);
            Long newUserNumLong = newUserNum.getLong();
            if (!resultMap.isEmpty() && !resultMap.get("dayN").isEmpty()) {
                String[] parts = resultMap.get("dayN").split("-");
                h5PayDayN = H5DayN.builder()
                        .rechargeMoney(new BigDecimal(parts[0]))
                        .rechargeNum(newUserNumLong)
                        .build();
            }
        }
        return h5PayDayN;
    }

    private String h5RechargeSql() {
        return """
                select
                a.game_id as id,
                a.game_id as game_id,
                max(a.game_name) as game_name,
                max(a.source_system) as source_system,
                sum(a.h5_buy_new_user_total_amount) as h5_buy_new_user_total_amount,
                sum(a.h5_buy_new_user_total_amount_num) as h5_buy_new_user_total_amount_num,
                round(if(sum(a.h5_buy_new_user_total_amount) > 0, sum(a.h5_nature_new_user_total_amount) / sum(a.h5_buy_new_user_total_amount), 0), 4) as h5_nature_new_user_total_amount_rate,
                sum(a.h5_nature_new_user_total_amount) as h5_nature_new_user_total_amount,
                sum(a.h5_nature_new_user_total_amount_num) as h5_nature_new_user_total_amount_num
                from ads_h5_game_day a
                """;
    }

    private String waterSql() {
        return """
                select
                game_id as id,
                source_system,
                game_id,
                max(game_name) as game_name,
                max(game_classify) as game_classify,
                max(amount) as amount,
                max(buy_amount) as buy_amount,
                max(nature_amount) as nature_amount
                from ads_everyday_water
                """;
    }

    /**
     * 流水临时sql
     *
     * @return String
     */
    private String waterTemplateSql() {
        return """
                select
                game_id,
                user_order_by,
                user_id,
                user_name,
                user_amount,
                amount_order_by,
                amount_post,
                amount_count,
                role_name,
                server_name,
                amount_num
                from ads_everyday_water
                """;
    }

    /**
     * DayN内部类
     */
    @Data
    @Builder
    public static class DayN {
        /**
         * 充值金额
         */
        private BigDecimal rechargeMoney;

        /**
         * 充值次数
         */
        private Long rechargeCount;

        /**
         * 充值人数
         */
        private Long rechargeNum;

        /**
         * 新用户复充人数
         */
        private Long rechargeAgainNum;
    }


    @Data
    @Builder
    public static class H5DayN {
        private BigDecimal rechargeMoney;
        private Long rechargeNum;
    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Builder
    public static class ShowRecharge {
        private BigDecimal h5NatureAmount;

        private Long h5NatureAmountNum;

        private BigDecimal h5BuyAmount;

        private Long h5BuyAmountNum;
    }
}
